@@ -136,7 +136,8 @@ sexprs.</p>")
 
    (defthm member-equal-of-list-fix
      (equal (member-equal a (list-fix x))
-            (list-fix (member-equal a x))))
+            (list-fix (member-equal a x)))
+     :hints(("Goal" :in-theory (enable member-equal))))
 
    (defthm list-fix-under-set-equiv
      (set-equiv (list-fix x)
@@ -265,7 +266,8 @@ sexprs.</p>")
      (local (defthm l0
               (implies (and (member-equal a x)
                             (member-equal k (4v-sexpr-vars a)))
-                       (member-equal k (4v-sexpr-vars-list x)))))
+                       (member-equal k (4v-sexpr-vars-list x)))
+              :hints(("Goal" :in-theory (enable member-equal)))))
 
      (defthm member-of-4v-sexpr-vars-list-when-subsetp
        (implies (and (subsetp-equal a b)
@@ -598,7 +600,7 @@ assumption that @('vars') are one-hot."
 
   :long "<p>How is this reduction accomplished?  Well, in the implementation of
 @(see 4v-shannon-expansion), reduced expressions are formed by using @(see
-sexpr-restrict) to assume that the variable being is first true, and then
+4v-sexpr-restrict) to assume that the variable being is first true, and then
 false.  Our approach is basically similar, and our new sexpr is essentially the
 following:</p>
 
@@ -771,7 +773,8 @@ assumption that the @('Ai') really are one-hot.</p>"
               (implies (and (all-equalp *4vf* (4v-sexpr-eval-list x env))
                             (member-equal a x))
                        (equal (4v-sexpr-eval a env)
-                              *4vf*))))
+                              *4vf*))
+              :hints(("Goal" :in-theory (enable member-equal)))))
 
      (local (defthm h3
               (implies (and (4v-onehot-list-p (4v-sexpr-eval-list sexprs env))
@@ -1039,8 +1042,8 @@ implement.</p>"
          (false-bindings  (hons-acons var (4vs-f) bindings)))
 
       (4vs-ite*-list-dumb (car vars)
-                     sexprs/bindings
-                     (4v-onehot-sexpr-list-prime-exec (cdr vars) false-bindings sexprs))))
+                          sexprs/bindings
+                          (4v-onehot-sexpr-list-prime-exec (cdr vars) false-bindings sexprs))))
 
   (local (in-theory (enable 4v-onehot-sexpr-list-prime-exec)))
 
@@ -1072,6 +1075,11 @@ implement.</p>"
            (implies (atom sexprs)
                     (not (4v-onehot-sexpr-list-prime-exec vars false-bindings sexprs)))))
 
+  (local (defthm l1a
+           (iff (4vs-ite*-list-dumb c as bs)
+                (consp as))
+           :hints(("Goal" :in-theory (enable 4vs-ite*-list-dumb)))))
+
   (local (defthm l1
            (equal (consp (4v-onehot-sexpr-list-prime-exec vars false-bindings sexprs))
                   (consp sexprs))))
