@@ -79,16 +79,13 @@
               (set-equiv (remove b a) (remove b c))))
   :hints(("Goal" :use ((:instance set-equiv-breakdown-cons)))))
 
-(defthm remove-remove
-  (equal (remove k (remove k x))
-         (remove k x))
-  :hints (("goal" :induct t)))
 
 (defthm remove-when-not-member
   (implies (not (member-equal k x))
            (equal (remove k x)
                   (append x nil)))
-  :hints(("Goal" :induct t)))
+  :hints(("Goal" :induct t
+          :in-theory (enable remove))))
 
 
 
@@ -175,7 +172,8 @@
                 (and (4v-sexpr-<= v (cdr (hons-assoc-equal k al2)))
                      (4v-sexpr-alist-<= al1 al2))))
   :hints (("Goal" :do-not-induct t)
-          (witness) (witness)))
+          (witness :ruleset (4v-sexpr-alist-<=-hons-assoc-equal-example
+                             4v-sexpr-alist-<=-witnessing))))
 
 (defthm 4v-sexpr-<=-restrict
   (implies (and (4v-sexpr-alist-<= a b)
