@@ -22,9 +22,9 @@
 (include-book "construction")
 (include-book "misc/hons-help" :dir :system)
 (include-book "centaur/aig/aig-vars-fast" :dir :system)
-(include-book "centaur/aig/base" :dir :system)
+(include-book "centaur/aig/aig-base" :dir :system)
 (local (include-book "arithmetic/top-with-meta" :dir :system))
-
+(local (include-book "std/lists/nthcdr" :dir :system))
 ;; Translating from Hons AIGs to aignets.
 
 ;; We need a memoization table so that we don't revisit AIG nodes we've already
