@@ -21,7 +21,7 @@
 (in-package "ACL2")
 (include-book "xdoc/top" :dir :system)
 (include-book "tools/bstar" :dir :system)
-(include-book "finite-set-theory/osets/sets" :dir :system)
+(include-book "std/osets/top" :dir :system)
 (local (include-book "ihs-extensions"))
 (local (include-book "std/lists/rev" :dir :system))
 (local (include-book "std/lists/append" :dir :system))
