@@ -22,7 +22,7 @@
 (include-book "misc/total-order" :dir :system)
 (include-book "std/lists/mfc-utils" :dir :system)
 (include-book "centaur/misc/introduce-var" :dir :system)
-(local (include-book "centaur/misc/equal-by-nths" :dir :system))
+(local (include-book "std/lists/nth" :dir :system))
 (local (include-book "array-lemmas"))
 (set-verify-guards-eagerness 2)
 
