@@ -28,6 +28,17 @@
 (include-book "cutil/deflist" :dir :system)
 (include-book "cutil/defmvtypes" :dir :system)
 
+(make-event
+
+; Disabling waterfall parallelism because this book allegedly uses memoization
+; while performing its proofs.  
+
+ (if (and (hons-enabledp state) 
+          (f-get-global 'parallel-execution-enabled state)) 
+     (er-progn (set-waterfall-parallelism nil)
+               (value '(value-triple nil)))
+   (value '(value-triple nil))))
+
 (cutil::deflist cons-listp (x)
                 (consp x)
                 :guard t
@@ -435,8 +446,8 @@ list-fix the path or similar.</p>"
     (declare (xargs :guard t))
     (b* (((when (atom instnames))
           mod)
-         (name1 (car instnames))
-         (occ   (cdr (hons-get name1 (occmap mod)))))
+         (name1  (car instnames))
+         (occ    (cdr (hons-get name1 (make-fast-alist (occmap mod))))))
       (and occ
            (follow-esim-path (cdr instnames) (gpl :op occ)))))
 
