@@ -35,6 +35,16 @@
 (local (include-book "esim-sexpr-support-thms"))
 (local (include-book "centaur/4v-sexpr/sexpr-advanced" :dir :system))
 
+(make-event
+
+; Disabling waterfall parallelism because this book allegedly uses memoization
+; while performing its proofs.  
+
+ (if (and (hons-enabledp state) 
+          (f-get-global 'parallel-execution-enabled state)) 
+     (er-progn (set-waterfall-parallelism nil)
+               (value '(value-triple nil)))
+   (value '(value-triple nil))))
 
 (local (in-theory (disable* sets::double-containment)))
 
@@ -304,16 +314,16 @@
          (no-duplicatesp-equal (alist-keys acc))))
 
   (defthm alist-keys-pattern-to-indices
-    (set-equivp (alist-keys (mv-nth 0 (pattern-to-indices pat acc idx)))
+    (set-equiv (alist-keys (mv-nth 0 (pattern-to-indices pat acc idx)))
                 (append (pat-flatten1 pat) (alist-keys acc)))
     :hints (("goal"
              :induct t
              :in-theory (enable pat-flatten1))
-            (witness :ruleset set-equivp-witnessing)))
+            (witness :ruleset set-equiv-witnessing)))
 
 
   (defthm alist-keys-esim-sexpr-indices-occs
-    (set-equivp (alist-keys (mv-nth 0 (esim-sexpr-indices-occs mod occs acc idx)))
+    (set-equiv (alist-keys (mv-nth 0 (esim-sexpr-indices-occs mod occs acc idx)))
                 (append (collect-signal-list
                          :o (occs-for-names occs mod))
                         (collect-signal-list
@@ -321,7 +331,7 @@
                         (alist-keys acc)))
     :hints (("goal" :induct t :do-not-induct t
              :in-theory (enable fal-extract))
-            (witness :ruleset set-equivp-witnessing))))
+            (witness :ruleset set-equiv-witnessing))))
 
 
 
@@ -409,8 +419,9 @@
 (defsection 4v-sexpr-vars-list
 
   (defthm 4v-sexpr-vars-list-append
-    (set-equivp (4v-sexpr-vars-list (append a b))
-                (append (4v-sexpr-vars-list a) (4v-sexpr-vars-list b))))
+    (set-equiv (4v-sexpr-vars-list (append a b))
+               (append (4v-sexpr-vars-list a) (4v-sexpr-vars-list b)))
+    :hints(("Goal" :in-theory (enable append))))
 
   (defthm not-member-4v-sexpr-vars-of-hons-assoc-equal
     (implies (not (member-equal v (4v-sexpr-vars-list (alist-vals al))))
@@ -541,7 +552,7 @@ the existing update functions without modification."
                          update-fns))
 
     (defthm esim-simplify-update-fns-alist-keys-same
-      (set-equivp (alist-keys (esim-simplify-update-fns mod update-fns))
+      (set-equiv (alist-keys (esim-simplify-update-fns mod update-fns))
                   (alist-keys update-fns)))
 
     (defthm esim-simplify-update-fns-4v-sexpr-vars-subset
@@ -560,7 +571,7 @@ the existing update functions without modification."
              (good-sexpr-varmap
               (mod-varmap mod)
               (esim-simplify-update-fns mod update-fns)))
-    :hints (("goal" :in-theory (e/d (subsetp-equal-trans2)
+    :hints (("goal" :in-theory (e/d (subsetp-trans2)
                                     (esim-simplify-update-fns-4v-sexpr-vars-subset))
              :use esim-simplify-update-fns-4v-sexpr-vars-subset))))
 
@@ -1210,7 +1221,7 @@ the existing update functions without modification."
                                         mod (alist-keys (occmap mod)))))
            :hints(("Goal" :in-theory (enable mod-varmap)
                    :do-not-induct t)
-                  (witness :ruleset subsetp-equal-witnessing))
+                  (witness :ruleset subsetp-witnessing))
            :otf-flg t))
 
        (verify-guards esim-sexpr-fnname
@@ -1226,7 +1237,7 @@ the existing update functions without modification."
                                    good-sexpr-varmap
                                    esim-sexpr-general-fnname
                                    4v-sexpr-restrict
-                                   subsetp-car-member-equal
+                                   subsetp-car-member
                                    hons-set-diff
                                    sexpr-fixpoints))))
          :otf-flg t)
@@ -1400,7 +1411,7 @@ the existing update functions without modification."
                                  occmap
                                  good-sexpr-varmap
                                  4v-sexpr-restrict
-                                 subsetp-car-member-equal
+                                 subsetp-car-member
                                  hons-set-diff
                                  sexpr-fixpoints)))
                (and stable-under-simplificationp
