@@ -1,8 +1,24 @@
-
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.  This program is distributed in the hope that it will be useful but
+; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+; more details.  You should have received a copy of the GNU General Public
+; License along with this program; if not, write to the Free Software
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
-
-
 (include-book "cutil/defaggregate" :dir :system)
 (include-book "tools/flag" :dir :system)
 (include-book "tools/pattern-match" :dir :system)
@@ -100,8 +116,8 @@
     `(cons ,(mk-constructor-aux (car tree))
            ,(mk-constructor-aux (cdr tree)))))
 
-(defun mk-constructor (basename tag names tree)
-  `(defund-inline ,basename ,names
+(defun mk-constructor (basename tag names tree notinline)
+  `(,(if notinline 'defund 'defund-inline) ,basename ,names
      (declare (xargs :guard t))
      ,(if tag
           `(cons ',tag
@@ -136,12 +152,12 @@
                :rule-classes (:rewrite :linear))
             (accessor-acl2-count-thms basename (cdr accs-alist))))))
 
-(defun defagg-fn (basename fields tag)
+(defun defagg-fn (basename fields tag notinline)
   (b* ((tree (list-to-tree fields))
        (accs-alist (mk-accs-alist basename tree))
        (?accs (strip-cars accs-alist)))
     `(defsection ,basename
-       ,(mk-constructor basename tag fields tree)
+       ,(mk-constructor basename tag fields tree notinline)
        ,@(mk-accessors accs-alist tag)
        ,@(accessors-of-constructor basename accs-alist fields fields)
        ,@(accessor-acl2-count-thms basename accs-alist)
@@ -160,9 +176,10 @@
                 (def-pattern-match-constructor
                   (x) ,basename (eq (tag x) ',tag) ,accs))))))
 
-(defmacro defagg (basename fields &key (tag 'nil tag-p))
+(defmacro defagg (basename fields &key (tag 'nil tag-p) notinline)
   (defagg-fn basename fields
-    (if tag-p tag (intern (symbol-name basename) "KEYWORD"))))
+    (if tag-p tag (intern (symbol-name basename) "KEYWORD"))
+    notinline))
 
 (logic)
 
@@ -173,7 +190,7 @@
 
 
 
-      
+
 
 ;; (defun da-accessors (name fields)
 ;;   (if (atom fields)
@@ -258,7 +275,7 @@
 ;;        (or (consp fields)
 ;;            (er hard 'defaggregate "There must be at least one field.~%"))
 ;;        (or (and (tuple-listp 2 require)
-;;                 (symbol-listp (strip-cars require)))     
+;;                 (symbol-listp (strip-cars require)))
 ;;            (er hard 'defaggregate ":require must be a list of (name requirement) tuples.~%"))
 ;;        (or (no-duplicatesp (strip-cars require))
 ;;            (er hard 'defaggregate "The names given to :require must be unique.~%"))
