@@ -1,20 +1,41 @@
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.  This program is distributed in the hope that it will be useful but
+; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+; more details.  You should have received a copy of the GNU General Public
+; License along with this program; if not, write to the Free Software
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
-
 (include-book "g-if")
 (include-book "g-primitives-help")
 (include-book "symbolic-arithmetic-fns")
 (include-book "eval-g-base")
-;(include-book "tools/with-arith5-help" :dir :system)
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
 (local (include-book "hyp-fix-logic"))
-;(local (allow-arith5-help))
 
 (local (in-theory (disable acl2::revappend-removal)))
 
-
-
+;; [Jared] I suspect we could get this easily working with implode/explode, but
+;; it seems easiest to just say, "this book is low level and wants to deal with
+;; coerce on its own."
+(local (in-theory (e/d (str::implode
+                        str::explode)
+                       (str::coerce-to-string-removal
+                        str::coerce-to-list-removal))))
 
 ;; ;; This brings if-then-elses up from atoms to the top level of a cons tree.
 ;; (defun propagate-ites-above-conses (x )
@@ -42,6 +63,10 @@
     (revappend-concrete (cdr a)
                         (gl-cons (mk-g-concrete (car a)) b))))
 
+(defthm no-deps-of-revappend-concrete
+  (implies (not (gobj-depends-on k p b))
+           (not (gobj-depends-on k p (revappend-concrete a b)))))
+
 (local
  (progn
    ;; (defthm gobjectp-revappend-concrete
@@ -65,7 +90,7 @@
 ;;   (cond ((or (atom x) (not (eq (tag x) :g-ite))) 0)
 ;;         (t (+ 1 (max (g-ite-depth (g-ite->then x))
 ;;                      (g-ite-depth (g-ite->else x)))))))
-    
+
 ;; (defthm g-ite-depth-of-g-ite->then
 ;;   (implies (eq (tag x) :g-ite)
 ;;            (< (g-ite-depth (g-ite->then x)) (g-ite-depth x)))
@@ -138,6 +163,13 @@
              (&
               (coerce-string (cdr x) (cons (code-char 0) pre) hyp))))))))
 
+(defthm deps-of-coerce-string
+  (implies (not (gobj-depends-on k p x))
+           (not (gobj-depends-on k p (coerce-string x pre hyp))))
+  :hints (("goal" :induct (coerce-string x pre hyp)
+           :expand ((coerce-string x pre hyp))
+           :in-theory (disable (:d coerce-string)))))
+
 
 (local
  (progn
@@ -302,6 +334,7 @@
                      (coerce (revappend pre (eval-g-base x env)) 'string)))
      :hints(("Goal" :in-theory (e/d* ( general-concrete-obj
                                        concrete-gobjectp-def
+                                       eval-g-base-list
                                        general-concretep-def)
                                      ((:definition coerce-string)
                                       member eval-g-base
@@ -347,6 +380,13 @@
       ((g-var &) (g-apply 'coerce (gl-list x 'list)))
       (& nil))))
 
+(defthm deps-of-coerce-list
+  (implies (not (gobj-depends-on k p x))
+           (not (gobj-depends-on k p (coerce-list x hyp))))
+  :hints (("goal" :induct  (coerce-list x hyp)
+           :expand ((coerce-list x hyp))
+           :in-theory (disable (:d coerce-list)))))
+
 
 ;; (local
 ;;  (defthm gobjectp-coerce-list
@@ -366,15 +406,16 @@
 (encapsulate nil
   (local (in-theory (disable member-equal)))
 
-  (local (defthm stringp-eval-g-base
-           (implies (and (not (general-concretep x))
-                         (not (g-ite-p x))
-                         (not (g-apply-p x))
-                         (not (g-var-p x)))
-                    (not (stringp (eval-g-base x env))))
-           :hints(("Goal" :in-theory (e/d ((:induction eval-g-base)
-                                           general-concretep-def))
-                   :induct (eval-g-base x env)
+  (local (defthm-gobj-flag
+           (defthm stringp-eval-g-base
+             (implies (and (not (general-concretep x))
+                           (not (g-ite-p x))
+                           (not (g-apply-p x))
+                           (not (g-var-p x)))
+                      (not (stringp (eval-g-base x env))))
+             :flag gobj)
+           :skip-others t
+           :hints(("Goal" :in-theory (e/d (general-concretep-def))
                    :expand ((:with eval-g-base (eval-g-base x env)))))))
 
 
@@ -385,7 +426,7 @@
                     (coerce (eval-g-base x env) 'list)))
     :hints(("Goal" :in-theory (e/d* (; (:ruleset g-correct-lemmas)
                                      eval-g-base-general-concrete-obj
-                                     eval-g-base)
+                                     eval-g-base eval-g-base-list)
                                     ((:definition coerce-list)
                                      eval-g-base-alt-def))
             :induct (coerce-list x hyp)
@@ -405,8 +446,8 @@
         (if (zp clk)
             (g-apply 'coerce (gl-list x y))
           (g-if ytest
-                (,gfn x ythen hyp clk)
-                (,gfn x yelse hyp clk))))
+                (,gfn x ythen hyp clk config bvar-db state)
+                (,gfn x yelse hyp clk config bvar-db state))))
        ((g-apply & &)
         (g-apply 'coerce (gl-list x y)))
        ((g-var &)
@@ -425,17 +466,23 @@
  coerce
  :hints `(("Goal" :in-theory (disable ,gfn))))
 
+(def-gobj-dependency-thm coerce
+  :hints `(("goal" :induct ,gcall
+            :expand (,gcall)
+            :in-theory (disable (:d ,gfn)))))
+
 (local
- (defthm eval-g-base-not-equal-list
-   (implies (and (not (general-concretep y))
-                 (not (g-ite-p y))
-                 (not (g-apply-p y))
-                 (not (g-var-p y)))
-            (not (equal (eval-g-base y env) 'list)))
-   :hints(("Goal" :in-theory (e/d ((:induction eval-g-base)
-                                   general-concretep-def)
+ (defthm-gobj-flag
+   (defthm eval-g-base-not-equal-list
+     (implies (and (not (general-concretep y))
+                   (not (g-ite-p y))
+                   (not (g-apply-p y))
+                   (not (g-var-p y)))
+              (not (equal (eval-g-base y env) 'list)))
+     :flag gobj)
+   :skip-others t
+   :hints(("Goal" :in-theory (e/d (general-concretep-def)
                                   (eval-g-base-alt-def))
-           :induct (eval-g-base y env)
            :expand ((:with eval-g-base (eval-g-base y env)))))))
 
 
@@ -462,8 +509,8 @@
                                      ; eval-g-base-g-apply-p
                                      eval-g-base-alt-def
                                      eval-g-base-not-equal-list))
-            :induct (,gfn x y hyp clk)
-            :expand ((,gfn x y hyp clk)))
+            :induct ,gcall
+            :expand (,gcall))
            (and stable-under-simplificationp
                 '(:in-theory (enable general-concrete-obj-correct
                                      eval-g-base-not-equal-list)))))
