@@ -1,15 +1,31 @@
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.  This program is distributed in the hope that it will be useful but
+; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+; more details.  You should have received a copy of the GNU General Public
+; License along with this program; if not, write to the Free Software
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
-
 (include-book "g-if")
 (include-book "g-primitives-help")
 (include-book "symbolic-arithmetic-fns")
 (include-book "eval-g-base")
-;(include-book "tools/with-arith5-help" :dir :system)
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
 (local (include-book "hyp-fix-logic"))
-;(local (allow-arith5-help))
 
 (defun g-floor-of-numbers (x y)
   (declare (xargs :guard (and (general-numberp x)
@@ -18,17 +34,24 @@
         (general-number-components x))
        ((mv yrn yrd yin yid)
         (general-number-components y)))
-    (if (and (eq (=-uu xrd '(t)) t)
-             (eq (=-uu yrd '(t)) t)
-             (eq (bfr-or (=-ss xin nil)
-                       (=-uu xid nil)) t)
-             (eq (bfr-or (=-ss yin nil)
-                       (=-uu yid nil)) t))
-        (mk-g-number (rlist-fix (floor-ss xrn yrn)))
+    (if (and (eq (bfr-=-uu xrd '(t)) t)
+             (eq (bfr-=-uu yrd '(t)) t)
+             (eq (bfr-or (bfr-=-ss xin nil)
+                       (bfr-=-uu xid nil)) t)
+             (eq (bfr-or (bfr-=-ss yin nil)
+                       (bfr-=-uu yid nil)) t))
+        (mk-g-number (rlist-fix (bfr-floor-ss xrn yrn)))
       (g-apply 'floor (gl-list x y)))))
 
 (in-theory (disable (g-floor-of-numbers)))
 
+(defthm deps-of-g-floor-of-numbers
+  (implies (and (not (gobj-depends-on k p x))
+                (not (gobj-depends-on k p y))
+                (general-numberp x)
+                (general-numberp y))
+           (not (gobj-depends-on k p (g-floor-of-numbers x y)))))
+
 ;; (local
 ;;  (defthm gobjectp-g-floor-of-numbers
 ;;    (implies (and (gobjectp x)
@@ -39,14 +62,14 @@
 
 (local (include-book "arithmetic/top-with-meta" :dir :system))
 
-;; (local (defthm not-integerp-floor-ss
+;; (local (defthm not-integerp-bfr-floor-ss
 ;;          (implies (and (bfr-listp a) (bfr-listp b))
-;;                   (not (integerp (floor-ss a b))))
-;;          :hints (("goal" :use ((:instance bfr-listp-floor-ss))
-;;                   :in-theory (e/d (bfr-listp) (bfr-listp-floor-ss))))))
+;;                   (not (integerp (bfr-floor-ss a b))))
+;;          :hints (("goal" :use ((:instance bfr-listp-bfr-floor-ss))
+;;                   :in-theory (e/d (bfr-listp) (bfr-listp-bfr-floor-ss))))))
 
-(local (add-bfr-fn-pat =-uu))
-(local (add-bfr-fn-pat =-ss))
+(local (add-bfr-fn-pat bfr-=-uu))
+(local (add-bfr-fn-pat bfr-=-ss))
 
 (local
  (defthm g-floor-of-numbers-correct
@@ -83,17 +106,22 @@
 ;;                                      gobj-fix-when-gobjectp
 ;;                                      (:rules-of-class :type-prescription :here)
 ;;                                      (:ruleset gl-wrong-tag-rewrites)))
-;;             :induct (,gfn i j hyp clk)
+;;             :induct (,gfn i j . ,params)
 ;;             :do-not-induct t
-;;             :expand ((,gfn i j hyp clk)
+;;             :expand ((,gfn i j . ,params)
 ;;                      (gobjectp (floor (gobj-fix i) (gobj-fix j)))))))
 
 (verify-g-guards
  floor
  :hints `(("goal" :in-theory
-           (disable* ,gfn 
+           (disable* ,gfn
                      (:rules-of-class :type-prescription :here)))))
 
+(def-gobj-dependency-thm floor
+  :hints `(("goal" :induct ,gcall
+            :expand (,gcall)
+            :in-theory (disable (:d ,gfn)))))
+
 (local (defthm floor-when-not-numberp
          (and (implies (not (acl2-numberp i))
                        (equal (floor i j) (floor 0 j)))
@@ -112,19 +140,18 @@
                               general-consp-cdr-correct-for-eval-g-base
                               boolean-listp
                               components-to-number-alt-def
-                              member-equal 
+                              member-equal
                               general-number-components-ev
                               general-concretep-def
-                              v2n-is-v2i-when-sign-nil
                               general-concretep-def
                               floor
                               hons-assoc-equal
                               rationalp-implies-acl2-numberp
                               (:rules-of-class :type-prescription :here))
                              ((:type-prescription bfr-eval)))
-     :induct (,gfn i j hyp clk)
+     :induct (,gfn i j . ,params)
      :do-not-induct t
-     :expand ((,gfn i j hyp clk)))
+     :expand ((,gfn i j . ,params)))
     (and stable-under-simplificationp
          (flag::expand-calls-computed-hint
           clause '(eval-g-base)))))
