@@ -1,19 +1,32 @@
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.  This program is distributed in the hope that it will be useful but
+; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+; more details.  You should have received a copy of the GNU General Public
+; License along with this program; if not, write to the Free Software
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
-
-(set-inhibit-warnings "theory")
-
 (include-book "g-if")
 (include-book "g-primitives-help")
 (include-book "symbolic-arithmetic-fns")
 (include-book "eval-g-base")
-;(include-book "tools/with-arith5-help" :dir :system)
 (local (include-book "symbolic-arithmetic"))
 (local (include-book "eval-g-base-help"))
 (local (include-book "hyp-fix-logic"))
-;(local (allow-arith5-help))
- 
-
+(set-inhibit-warnings "theory")
 
 (def-g-fn lognot
   `(let ((x i))
@@ -24,8 +37,8 @@
           (if (zp clk)
               (g-apply 'lognot (gl-list x))
             (g-if test
-                  (,gfn then hyp clk)
-                  (,gfn else hyp clk))))
+                  (,gfn then . ,params)
+                  (,gfn else . ,params))))
          ((g-apply & &)
           (g-apply 'lognot (gl-list x)))
          ((g-concrete obj)
@@ -38,10 +51,10 @@
                 (break-g-number num))
                ((mv intp intp-known)
                 (if (equal rd '(t))
-                    (mv (bfr-or (=-ss in nil) (=-uu id nil)) t)
+                    (mv (bfr-or (bfr-=-ss in nil) (bfr-=-uu id nil)) t)
                   (mv nil nil))))
             (if intp-known
-                (mk-g-number (lognot-s (bfr-ite-bss-fn intp rn nil)))
+                (mk-g-number (bfr-lognot-s (bfr-ite-bss-fn intp rn nil)))
               (g-apply 'lognot (gl-list x)))))
          (& -1)))))
 
@@ -54,14 +67,19 @@
 ;; (def-gobjectp-thm lognot
 ;;   :hints `(("Goal" :in-theory (e/d ()
 ;;                                    ((:definition ,gfn) lognot))
-;;             :induct (,gfn i hyp clk)
-;;             :expand ((,gfn i hyp clk)
+;;             :induct (,gfn i . ,params)
+;;             :expand ((,gfn i . ,params)
 ;;                      (:free (x) (gobjectp (- x)))))))
 
 (verify-g-guards
  lognot
  :hints `(("Goal" :in-theory (disable ,gfn))))
 
+(def-gobj-dependency-thm lognot
+  :hints `(("goal" :induct ,gcall
+            :expand (,gcall)
+            :in-theory (disable (:d ,gfn)))))
+
 (local
  (progn
    (defthm lognot-non-acl2-numberp
@@ -81,7 +99,7 @@
                                      general-number-components-ev
                                      general-numberp-eval-to-numberp
                                      lognot))
-             :induct (,gfn i hyp clk)
-             :expand ((,gfn i hyp clk)
+             :induct (,gfn i . ,params)
+             :expand ((,gfn i . ,params)
                       (:with eval-g-base (eval-g-base i env))))))
 
