@@ -1,12 +1,28 @@
-
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.  This program is distributed in the hope that it will be useful but
+; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+; more details.  You should have received a copy of the GNU General Public
+; License along with this program; if not, write to the Free Software
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
-
 (include-book "gtype-thms")
 (include-book "general-objects")
 
 
-
 (defthmd generic-geval-g-boolean
   (implies (g-boolean-p x)
            (equal (generic-geval x env)
@@ -49,6 +65,13 @@
   :hints (("goal" :in-theory (enable generic-geval)))
   :rule-classes ((:rewrite :backchain-limit-lst 0)))
 
+(defthm pbfr-depends-on-of-general-boolean-value
+  (implies (and (not (gobj-depends-on k p x))
+                (general-booleanp x))
+           (not (pbfr-depends-on k p (general-boolean-value x))))
+  :hints(("Goal" :in-theory (enable general-booleanp general-boolean-value
+                                    booleanp))))
+
 
 (in-theory (disable general-booleanp general-boolean-value))
 
@@ -67,11 +90,11 @@
 
 (defthm boolean-listp-n2v
   (boolean-listp (n2v n))
-  :hints(("Goal" :in-theory (enable n2v))))
+  :hints(("Goal" :in-theory (e/d (n2v bfr-ucons) (logcar logcdr)))))
 
 (defthm boolean-listp-i2v
   (boolean-listp (i2v n))
-  :hints(("Goal" :in-theory (enable i2v))))
+  :hints(("Goal" :in-theory (e/d (i2v bfr-scons) (logcar logcdr)))))
 
 (defthm number-to-components-boolean-listps
   (and (boolean-listp (mv-nth 0 (number-to-components n)))
@@ -81,39 +104,33 @@
 
 
 
-(defthm v2n-us
-  (implies (natp n)
-           (equal (v2n (n2v n)) n))
-  :hints (("goal" :in-theory (disable floor))))
+;; (defthm v2n-us
+;;   (implies (natp n)
+;;            (equal (v2n (n2v n)) n))
+;;   :hints (("goal" :in-theory (disable floor))))
 
-(defthm v2n-non-natp
-  (implies (not (natp n))
-           (equal (v2n (n2v n)) 0)))
+;; (defthm v2n-non-natp
+;;   (implies (not (natp n))
+;;            (equal (v2n (n2v n)) 0)))
 
-(defthm acl2-numberp-v2n
-  (and (acl2-numberp (v2n x))
-       (rationalp (v2n x))
-       (integerp (v2n x))
-       (natp (v2n x)))
-  :rule-classes (:rewrite :type-prescription))
+;; (defthm acl2-numberp-v2n
+;;   (and (acl2-numberp (v2n x))
+;;        (rationalp (v2n x))
+;;        (integerp (v2n x))
+;;        (natp (v2n x)))
+;;   :rule-classes (:rewrite :type-prescription))
 
 
 (defthm number-to-components-components-to-number
   (b* (((mv rnum rden inum iden)
         (number-to-components n)))
     (implies (acl2-numberp n)
-             (and (equal (components-to-number
-                          (v2i rnum)
-                          (v2n rden)
-                          (v2i inum)
-                          (v2n iden))
-                         n)
-                  (equal (components-to-number
-                          (v2i (bfr-eval-list rnum env))
-                          (v2n (bfr-eval-list rden env))
-                          (v2i (bfr-eval-list inum env))
-                          (v2n (bfr-eval-list iden env)))
-                         n))))
+             (equal (components-to-number
+                     (bfr-list->s rnum env)
+                     (bfr-list->u rden env)
+                     (bfr-list->s inum env)
+                     (bfr-list->u iden env))
+                    n)))
   :hints (("goal" :in-theory (enable components-to-number-alt-def))))
 
 
@@ -138,9 +155,9 @@
            (mv-let (rn rd in id)
              (general-number-components a)
              (flet ((uval (n env)
-                          (v2n (bfr-eval-list n (car env))))
+                          (bfr-list->u n (car env)))
                     (sval (n env)
-                          (v2i (bfr-eval-list n (car env)))))
+                          (bfr-list->s n (car env))))
                (equal (generic-geval a env)
                       (components-to-number
                        (sval rn env)
@@ -153,6 +170,18 @@
 (in-theory (disable general-number-components
                     general-numberp))
 
+(defthm pbfr-depends-on-of-general-number-components
+  (implies (and (general-numberp x)
+                (not (gobj-depends-on k p x)))
+           (b* (((mv rn rd in id) (general-number-components x)))
+             (and (not (pbfr-list-depends-on k p rn))
+                  (not (pbfr-list-depends-on k p rd))
+                  (not (pbfr-list-depends-on k p in))
+                  (not (pbfr-list-depends-on k p id)))))
+  :hints(("Goal" :in-theory (enable general-number-components)
+          :expand ((gobj-depends-on k p x)
+                   (general-numberp x)))))
+
 
 
 (local (defthm nth-open-when-constant-idx
@@ -160,7 +189,7 @@
                   (equal (nth n x)
                          (if (zp n) (car x)
                            (nth (1- n) (cdr x)))))))
-                       
+
 
 (defthm general-numberp-of-atom
   (implies (not (consp x))
@@ -227,6 +256,17 @@
   :hints (("goal" :in-theory (enable mk-g-concrete g-concrete g-concrete->obj)))
   :rule-classes (:rewrite :linear))
 
+(defthm gobj-depends-on-of-general-consp
+  (implies (and (not (gobj-depends-on k p x))
+                (general-consp x))
+           (and (not (gobj-depends-on k p (general-consp-car x)))
+                (not (gobj-depends-on k p (general-consp-cdr x)))))
+  :hints(("Goal" :in-theory (enable general-consp
+                                    general-consp-car
+                                    general-consp-cdr
+                                    g-keyword-symbolp-def))))
+
+
 (in-theory (disable general-consp general-consp-car general-consp-cdr))
 
 
@@ -338,7 +378,7 @@
 
 (verify-guards
  general-concrete-obj)
-    
+
 
 ;; (defun general-concretep (x)
 ;;   (declare (xargs :guard (gobjectp x)))
@@ -358,14 +398,16 @@
   (implies (concrete-gobjectp x)
            (equal (generic-geval x env)
                   x))
-  :hints(("Goal" :in-theory (enable concrete-gobjectp-def)))
+  :hints(("Goal" :in-theory (enable concrete-gobjectp
+                                    gobject-hierarchy-lite
+                                    generic-geval)))
   :rule-classes ((:rewrite :backchain-limit-lst 0)))
 
 (defthmd general-concrete-obj-correct
   (implies (general-concretep x)
            (equal (generic-geval x env)
                   (general-concrete-obj x)))
-  :hints(("Goal" 
+  :hints(("Goal"
           :induct (general-concrete-obj x)
 
           :expand ((general-concretep x)
@@ -467,7 +509,7 @@
 
 
 
-;; (local 
+;; (local
 ;;  (defthm g-concrete-p-impl-not-general-concretep-car
 ;;    (implies (g-concrete-p x)
 ;;             (not (general-concretep (car x))))
@@ -506,9 +548,9 @@
            (b* (((mv rn rd in id)
                  (general-number-components x)))
              (flet ((uval (n env)
-                          (v2n (bfr-eval-list n (car env))))
+                          (bfr-list->u n (car env)))
                     (sval (n env)
-                          (v2i (bfr-eval-list n (car env)))))
+                          (bfr-list->s n (car env))))
                (components-to-number (sval rn env)
                                      (uval rd env)
                                      (sval in env)
@@ -521,18 +563,19 @@
                (generic-geval (g-ite->then x) env)
              (generic-geval (g-ite->else x) env)))
           ((g-apply-p x)
-           (generic-geval-ev
-            (cons (g-apply->fn x)
-                  (kwote-lst
-                   (generic-geval (g-apply->args x) env)))
-            nil))
+           (and (not (eq (g-apply->fn x) 'quote))
+                (generic-geval-ev
+                 (cons (g-apply->fn x)
+                       (kwote-lst
+                        (generic-geval-list (g-apply->args x) env)))
+                 nil)))
           (t
            (cdr (hons-assoc-equal (g-var->name x) (cdr env))))))
   :hints (("goal" ;; :induct (generic-geval x env)
            :in-theory (e/d** (general-concretep-def
                               (:induction generic-geval)
                               general-numberp general-booleanp
-                              general-consp eq 
+                              general-consp eq
                               atom
                               acl2::cons-car-cdr
                               concrete-gobjectp-def
@@ -549,7 +592,10 @@
                               hons-get))
            :do-not-induct t
            :expand ((generic-geval x env))))
-  :rule-classes :definition)
+  :rule-classes
+  ((:definition :clique (generic-geval generic-geval-list)
+    :controller-alist ((generic-geval t nil)
+                       (generic-geval-list t nil)))))
 
 
 (in-theory (disable generic-geval-alt-def))
@@ -786,10 +832,10 @@
              (general-number-components x)
              (equal (general-concrete-obj x)
                     (components-to-number-fn
-                     (v2i (bfr-eval-list rn (car env)))
-                     (v2n (bfr-eval-list rd (car env)))
-                     (v2i (bfr-eval-list in (car env)))
-                     (v2n (bfr-eval-list id (car env)))))))
+                     (bfr-list->s rn (car env))
+                     (bfr-list->u rd (car env))
+                     (bfr-list->s in (car env))
+                     (bfr-list->u id (car env))))))
   :hints(("Goal" :in-theory (enable ;general-concretep-def
                              general-numberp
                              general-concrete-obj
