@@ -1,14 +1,31 @@
-
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.  This program is distributed in the hope that it will be useful but
+; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+; more details.  You should have received a copy of the GNU General Public
+; License along with this program; if not, write to the Free Software
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
-
 (include-book "gify-clause-proc")
-
+(include-book "std/misc/two-nats-measure" :dir :system)
 (local (include-book "gtype-thms"))
 (local (include-book "gobjectp-thms"))
 (local (include-book "general-object-thms"))
 (local (include-book "hyp-fix-logic"))
-(include-book "std/misc/two-nats-measure" :dir :system)
+
 
 ;; These events are included here redundantly to avoid having to include all
 ;; the above (locally-included) books everywhere we want a GL clause processor.
@@ -68,6 +85,26 @@
          (cons (generic-geval x env)
                (generic-geval y env))))
 
+(defthm generic-geval-list-gl-cons
+  (equal (generic-geval-list (gl-cons x y) env)
+         (cons (generic-geval x env)
+               (generic-geval-list y env)))
+  :hints(("Goal" :expand ((:free (x) (generic-geval-list (cons x y) env))))))
+
+(defthm generic-geval-list-atom
+  (implies (not (consp x))
+           (equal (generic-geval-list x env) nil))
+  :hints(("Goal" :expand ((generic-geval-list x env))))
+  :rule-classes ((:rewrite :backchain-limit-lst 0)))
+
+(defthm generic-geval-g-apply
+  (implies (not (equal fn 'quote))
+           (equal (generic-geval (g-apply fn args) env)
+                  (generic-geval-ev (cons fn (kwote-lst (generic-geval-list args env)))
+                                    nil)))
+  :hints (("goal" :expand ((generic-geval (g-apply fn args) env))
+           :in-theory (enable generic-geval-apply))))
+
 (defthm generic-geval-nil
   (equal (generic-geval nil env) nil))
 
@@ -76,10 +113,11 @@
            (equal (generic-geval x env) x))
   :rule-classes ((:rewrite :backchain-limit-lst 0)))
 
-(defthm generic-geval-g-apply
-  (equal (generic-geval (g-apply fn args) env)
-         (generic-geval-ev (cons fn (kwote-lst (generic-geval args env)))
-                           nil)))
+;; (defthm generic-geval-g-apply
+;;   (implies (not (eq fn 'quote))
+;;            (equal (generic-geval (g-apply fn args) env)
+;;                   (generic-geval-ev (cons fn (kwote-lst (generic-geval-list args env)))
+;;                                     nil))))
 
 ;; (defthm generic-geval-gobj-fix
 ;;   (equal (generic-geval (gobj-fix x) env)
@@ -120,7 +158,7 @@
 ;; (add-to-ruleset! g-gobjectp-lemmas '(g-if-gobjectp-meta-correct
 ;;                                      gobjectp-g-if-marker
 ;;                                      g-or-gobjectp-meta-correct
-;;                                      gobjectp-g-or-marker 
+;;                                      gobjectp-g-or-marker
 ;;                                      gobjectp-g-apply
 ;;                                      gobjectp-gobj-fix
 ;;                                      gobjectp-g-test-marker
@@ -146,6 +184,8 @@
   '(generic-geval-gl-cons
     generic-geval-nil
     generic-geval-g-apply
+    generic-geval-list-gl-cons
+    generic-geval-list-atom
     mk-g-ite-correct
     mk-g-boolean-correct
     mk-g-concrete-correct
@@ -169,7 +209,11 @@
                                  bfr-and-of-nil
                                  bfr-or-of-t
                                  (g-keyword-symbolp)
-                                 gl-aside gl-error-is-nil))
+                                 gl-aside gl-error-is-nil
+                                 gobj-depends-on-of-g-if-marker
+                                 gobj-depends-on-of-g-or-marker
+                                 gobj-depends-on-g-if-meta
+                                 gobj-depends-on-g-or-meta))
 
 ;; (defthm bfr-fix-x-is-x-when-bfr-p
 ;;   (implies (bfr-p x)
