@@ -1,7 +1,24 @@
-
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.  This program is distributed in the hope that it will be useful but
+; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+; more details.  You should have received a copy of the GNU General Public
+; License along with this program; if not, write to the Free Software
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
-
 (include-book "tools/bstar" :dir :system)
 (include-book "defapply")
 (include-book "misc/hons-help" :dir :system)
@@ -30,7 +47,7 @@
        acc
      (collect-fns-lst (cdr lst) clique
                       (collect-fns (car lst) clique acc)))))
-          
+
 
 (defun clique-bodies (clique world)
   (if (atom clique)
@@ -101,13 +118,13 @@
                    (else (gify-term (cadddr x) fn)))
               `(g-if ,test ,then ,else))))
          ((consp (car x))
-          (er hard 'gify-term 
+          (er hard 'gify-term
               "We expect lambdas to all have been factored out of functions to
 be gified.  If not, implement this case.  Culprit (in function ~x0): ~x1~%"
               fn (car x)))
          (t (cons (gl-fnsym (car x))
                   (append (gify-term-list (cdr x) fn)
-                          '(hyp clk))))))
+                          '(hyp clk config bvar-db state))))))
  (defun gify-term-list (x fn)
    (if (endp x)
        nil
@@ -168,13 +185,15 @@ be gified.  If not, implement this case.  Culprit (in function ~x0): ~x1~%"
          (gbody (gify-body body (car fns)))
          (formals (wgetprop fn 'formals))
          (recursivep (wgetprop top-fn 'recursivep))
-         (guards `(natp clk))
+         (guards `(and (natp clk)
+                       (glcp-config-p config)))
          (name (gl-fnsym (if endp top-fn fn)))
          (defun
-           `(defun ,name ,(append formals '(hyp clk))
+           `(defun ,name ,(append formals '(hyp clk config bvar-db state))
               (declare
                (xargs
                 :guard ,guards
+                :stobjs (bvar-db state)
                 :verify-guards nil
                 . ,(and recursivep
                         `(:measure
@@ -183,7 +202,7 @@ be gified.  If not, implement this case.  Culprit (in function ~x0): ~x1~%"
                                   `(:hints (("goal" :in-theory
                                              (e/d** ((:ruleset
                                                       clk-measure-rules))))))))))
-               (ignorable hyp clk . ,formals))
+               (ignorable hyp clk config bvar-db state . ,formals))
                 ,(if endp
                      (let ((clkbody `(if (zp clk)
                                          (prog2$
@@ -209,7 +228,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
         (gify-factored-fns top-fn (cdr fns) (1+ idx) world
                            (cons defun defuns))))))
 
-                           
+
 
 (defmacro factored-fns (fn)
   `(cdr (assoc-eq ,fn (table-alist 'factored-fns world))))
@@ -232,7 +251,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
     (if (and recp (<= 2 (length defuns)))
         `(mutual-recursion . ,defuns)
       `(progn . ,(reverse defuns)))))
-                     
+
 
 (defun gify-stub (fn world)
   (let* ((formals (wgetprop fn 'formals))
@@ -241,8 +260,10 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
                       (wgetprop fn 'non-executablep)
                       (assoc-eq fn (table-alist 'do-not-execute
                                                 world)))))
-    `(defun ,name (,@formals hyp clk)
-       (declare (xargs :guard (natp clk)
+    `(defun ,name (,@formals hyp clk config bvar-db state)
+       (declare (xargs :guard (and (natp clk)
+                                   (glcp-config-p config))
+                       :stobjs (bvar-db state)
                        :verify-guards nil))
        ,(if nonexec
             `(g-apply ',fn (gl-list . ,formals))
@@ -282,7 +303,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
 
 
 
-            
+
 
 ;; (defun gobjectp-thmname (fn)
 ;;   (incat 'gl-thm::foo "GOBJECTP-" (symbol-name fn)))
@@ -298,7 +319,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
 ;;          (endp (endp (cdr fns)))
 ;;          (name (gl-fnsym (if endp top-fn fn)))
 ;;          (gobjectp-thm `(,name
-;;                          (gobjectp (,name ,@formals hyp clk))
+;;                          (gobjectp (,name ,@formals hyp clk config bvar-db state))
 ;;                          :name ,(gobjectp-thmname name))))
 ;;       (if endp
 ;;           (cons gobjectp-thm entries)
@@ -321,10 +342,10 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
 ;;          (thmname (gobjectp-thmname name))
 ;;          (formals (wgetprop fn 'formals)))
 ;;       (cons `(defthm ,thmname
-;;                (gobjectp (,name ,@formals hyp clk)))
+;;                (gobjectp (,name ,@formals hyp clk config bvar-db state)))
 ;;             (gobjectp-redundant-top-thms (cdr clique) world)))))
-         
-         
+
+
 
 
 (defmacro defname (name)
@@ -343,9 +364,9 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
 ;;          (defname (defname name))
 ;;          (gobjectp-thm
 ;;           `(defthm ,gobjectp-name
-;;              (gobjectp (,name ,@formals hyp clk))
+;;              (gobjectp (,name ,@formals hyp clk config bvar-db state))
 ;;              :hints (("goal" :expand ((:with (:definition ,defname)
-;;                                              (,name ,@formals hyp clk)))
+;;                                              (,name ,@formals hyp clk config bvar-db state)))
 ;;                       :do-not '(preprocess)))))
 ;;          (gobjectp-thm (if endp gobjectp-thm `(local ,gobjectp-thm))))
 ;;       (if endp
@@ -530,7 +551,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
 
 
 
-(defun g-factored-fns-verify-guards (top-fn fns world) 
+(defun g-factored-fns-verify-guards (top-fn fns world)
  (if (endp fns)
       (er hard 'g-factored-fns-verify-guards
           "Empty list of functions; top: ~x0~%" top-fn)
@@ -554,7 +575,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
           (cons `(:type-prescription ,(car clique))
                 (type-prescriptions-of-mv-fns (cdr clique) world))
         (type-prescriptions-of-mv-fns (cdr clique) world)))))
-          
+
 
 
 (defun g-guards-fns-and-deps1 (deps world)
@@ -588,7 +609,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
                                  zp natp)
                                 ((immediate-force-modep) not))))
        . ,(g-guards-fns-and-deps1 deps world))))
-           
+
 
 
 
@@ -622,7 +643,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
          (name (gl-fnsym fn))
          (g-correct-thm `(,name
                           (implies (bfr-eval hyp (car env))
-                                   (equal (,ev (,name ,@formals hyp clk) env)
+                                   (equal (,ev (,name ,@formals hyp clk config bvar-db state) env)
                                           (,fn . ,(eval-list-env ev formals))))
                           . ,(if endp
                                  `(:name ,(correct-thmname name))
@@ -662,10 +683,10 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
          (g-correct-thm
           `(defthm ,g-correct-name
              (implies (bfr-eval hyp (car env))
-                      (equal (,ev (,name ,@formals hyp clk) env)
+                      (equal (,ev (,name ,@formals hyp clk config bvar-db state) env)
                              (,fn . ,(eval-list-env ev formals))))
              :hints (("goal" :expand
-                      ((,name ,@formals hyp clk))
+                      ((,name ,@formals hyp clk config bvar-db state))
                       ,@(and endp definedp
                              `(:in-theory (e/d ()
                                                ,(factored-fn-correct-thms
@@ -789,7 +810,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
                                              `(:expand (,term)))))))
                                (and stable-under-simplificationp
                                     '(:expand ((:free (x) (hide x))))))))
-                      
+
                       ;; BOZO. This is an ugly sort of desperation move. Blech.
                       ;; (and stable-under-simplificationp
 ;;                            '(:in-theory (union-theories
@@ -857,7 +878,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
       (if (hons-get-any clique al)
           (remove-cliques-in-al (cdr fns) al world)
         (cons fn (remove-cliques-in-al (cdr fns) al world))))))
-         
+
 
 (defun lambda-for-apply-stub-fi (oldfns newfns world)
   `(lambda (f args)
@@ -897,7 +918,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
 ;;          (applyfns
 ;;           (cdr (assoc apply (table-alist 'g-apply-table world))))
 ;;          (oldapplyfns
-;;           (cdr (assoc oldapply (table-alist 'g-apply-table world))))) 
+;;           (cdr (assoc oldapply (table-alist 'g-apply-table world)))))
 ;;   `(:functional-instance
 ;;     ,thmname
 ;;     ,@(if (eq oldapply 'apply-stub)
@@ -909,15 +930,16 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
 
 
 (defun eval-g-fi (eval oldeval thmname world)
-  (b* (((list ?new-appalist new-ev new-evlst)
+  (b* (((list eval-list new-ev new-evlst)
         (cdr (assoc eval (table-alist 'eval-g-table world))))
-       ((list ?old-appalist old-ev old-evlst)
+       ((list oldeval-list old-ev old-evlst)
         (cdr (assoc oldeval (table-alist 'eval-g-table world)))))
     `(:functional-instance
       ,thmname
       (,old-ev ,new-ev)
       (,old-evlst ,new-evlst)
-      (,oldeval ,eval))))
+      (,oldeval ,eval)
+      (,oldeval-list ,eval-list))))
 
 (defconst *eval-g-prove-f-i-template*
   '(:computed-hint-replacement
@@ -930,11 +952,12 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
     ((:functional-instance
       _theorem_
       (_oldgeval_ _newgeval_)
+      (_oldgeval_-list _newgeval_-list)
       (_oldgeval_-ev _newgeval_-ev)
       (_oldgeval_-ev-lst _newgeval_-ev-lst)))
     :in-theory (e/d**
-                (nth-of-_oldgeval_-ev-concrete-lst
-                 nth-of-_newgeval_-ev-concrete-lst
+                (;; nth-of-_oldgeval_-ev-concrete-lst
+                 ;; nth-of-_newgeval_-ev-concrete-lst
                  acl2::car-to-nth-meta-correct
                  acl2::nth-of-cdr
                  acl2::list-fix-when-true-listp
@@ -944,11 +967,12 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
                  _newgeval_-apply-agrees-with-_newgeval_-ev-rev
                  _oldgeval_-ev-rules
                  _newgeval_-ev-rules
-                 _newgeval_-ev-constraint-0
-                 _oldgeval_-ev-constraint-0
+                 _newgeval_-ev-of-fncall-args
+                 _oldgeval_-ev-of-fncall-args
                  car-cons cdr-cons nth-0-cons (nfix)))
-    :expand ((_oldgeval_ x env)
-             (_newgeval_ x env))
+    :expand (; (_oldgeval_ x env)
+             (:with _newgeval_ (_newgeval_ x env))
+             (:with _newgeval_-list (_newgeval_-list x env)))
     :do-not-induct t))
 
 (defun eval-g-prove-f-i-fn (eval oldeval thmname)
@@ -990,7 +1014,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
      ;;                   `(:theorem (equal (,oldeval x env)
      ;;                                     (,oldeval x env)))
      ;;                   world))
-               
+
      ;;           :do-not '(preprocess simplify)
      ;;           :in-theory nil))
      ;; :rule-classes nil)))
@@ -1006,12 +1030,17 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
   (incat 'gl-thm::foo (symbol-name thm) "-FOR-" (symbol-name eval)))
 
 (defun eval-g-functional-instance-fn (eval oldeval thmname world)
-  (let* ((apply (caddr (assoc eval (table-alist 'eval-g-table world))))
-         (oldapply (caddr (assoc oldeval (table-alist 'eval-g-table world))))
-         (thmbody (wgetprop thmname 'theorem))
-         (subst-body (subst-fns-term thmbody (list (cons oldeval eval)
-                                                   (cons oldapply apply))))
-         (newthmname (f-i-thmname thmname eval)))
+  (b* (((list neweval-list new-ev new-evlst new-apply)
+        (cdr (assoc eval (table-alist 'eval-g-table world))))
+       ((list oldeval-list old-ev old-evlst old-apply)
+        (cdr (assoc oldeval (table-alist 'eval-g-table world))))
+       (thmbody (wgetprop thmname 'theorem))
+       (subst-body (subst-fns-term thmbody (list (cons oldeval eval)
+                                                 (cons old-apply new-apply)
+                                                 (cons oldeval-list neweval-list)
+                                                 (cons old-ev new-ev)
+                                                 (cons old-evlst new-evlst))))
+       (newthmname (f-i-thmname thmname eval)))
     `(defthm ,newthmname
        ,subst-body
        :hints (("goal" :in-theory (theory 'minimal-theory)
@@ -1019,8 +1048,8 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
 
 (defmacro eval-g-functional-instance (thmname eval oldeval)
   `(make-event (eval-g-functional-instance-fn ',eval ',oldeval ',thmname (w state))))
-    
-  
+
+
 
 
 (defun correctness-lemmas-for-new-apply11
@@ -1035,7 +1064,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
                ((mv thms done-finsts)
                 (if (member thmeval done-finsts)
                     (mv thms done-finsts)
-                  (mv (cons `(local (eval-g-prove-f-i 
+                  (mv (cons `(local (eval-g-prove-f-i
                                      ,(incat 'gl-thm::foo
                                              (symbol-name eval)
                                              "-IS-FUNCTIONAL-INSTANCE-OF-"
@@ -1054,7 +1083,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
          (cdr alist) eval world done-finsts thms (cons thmbase theory))))))
 
 
-  
+
 (defun correctness-lemmas-for-new-apply1 (alist eval world
                                                 done-finsts thms theory)
   (if (atom alist)
@@ -1182,7 +1211,7 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
              `(in-theory (disable ,thmname))
              `(table preferred-defs ',fn ',thmname)
              (make-unnorm-preferred-defs1 (cdr fns) world)))))
-           
+
 
 (defun make-unnorm-preferred-defs (fns world)
   `(progn . ,(make-unnorm-preferred-defs1 fns world)))
@@ -1225,7 +1254,9 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
        (new-fns (set-difference-eq
                  (collect-fn-deps fns world)
                  '(acl2::return-last)))
-       (apply-fns (union-eq new-fns (prev-apply-fns world))))
+       (apply-fns (union-eq new-fns
+                            (prev-apply-fns world)
+                            (strip-cars (table-alist 'gl-function-info world)))))
     `(encapsulate nil
        (logic)
        (local (table acl2::theory-invariant-table nil nil :clear))
@@ -1241,8 +1272,8 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
        (make-event (gify-fns-and-deps ',new-fns (w state)))
        (make-event (flagify-g-fns-and-deps ',new-fns (w state)))
 
-       
-       
+
+
        ;; (make-event (gobjectp-fns-and-deps ',new-fns (w state)))
        (make-event (g-guards-fns-and-deps ',new-fns (w state)))
 
@@ -1267,10 +1298,14 @@ Warning: Clock ran out in ~x0~%" ',(gl-fnsym top-fn))
                                                       state))))
 
 (defun make-geval-fn (geval new-fns state)
-  (declare (xargs :stobjs state))
+  (declare (xargs :stobjs state :mode :program))
   (b* ((world (w state))
-       (new-fns (collect-fn-deps new-fns world))
-       (fns (union-eq new-fns (prev-apply-fns world)))
+       (new-fns (set-difference-eq
+                 (collect-fn-deps new-fns world)
+                 '(acl2::return-last)))
+       (fns (union-eq new-fns
+                      (strip-cars (table-alist 'gl-function-info world))
+                      (prev-apply-fns world)))
        ;; (ap (incat geval (symbol-name geval) "-APPLY"))
        )
     `(encapsulate nil
