@@ -1,7 +1,24 @@
-
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.  This program is distributed in the hope that it will be useful but
+; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+; more details.  You should have received a copy of the GNU General Public
+; License along with this program; if not, write to the Free Software
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
-
 (include-book "tools/flag" :dir :system)
 (include-book "tools/bstar" :dir :system)
 
@@ -203,7 +220,7 @@ passed to ~x2 in that theorem.~%"
   `(,(gl-fnsym fn) ,@args))
 
 (defmacro glc (fn &rest args)
-  `(,(gl-fnsym fn) ,@args t 1000000))
+  `(,(gl-fnsym fn) ,@args hyp clk config bvar-db state))
 
 
 
@@ -230,5 +247,5 @@ passed to ~x2 in that theorem.~%"
            g-var g-var-p g-var->name))))
 
 
-                  
+
 
