@@ -1,19 +1,33 @@
+; GL - A Symbolic Simulation Framework for ACL2
+; Copyright (C) 2008-2013 Centaur Technology
+;
+; Contact:
+;   Centaur Technology Formal Verification Group
+;   7600-C N. Capital of Texas Highway, Suite 300, Austin, TX 78731, USA.
+;   http://www.centtech.com/
+;
+; This program is free software; you can redistribute it and/or modify it under
+; the terms of the GNU General Public License as published by the Free Software
+; Foundation; either version 2 of the License, or (at your option) any later
+; version.  This program is distributed in the hope that it will be useful but
+; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+; more details.  You should have received a copy of the GNU General Public
+; License along with this program; if not, write to the Free Software
+; Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
+;
+; Original author: Sol Swords <sswords@centtech.com>
 
 (in-package "GL")
-
 (include-book "general-objects")
-
-(local (include-book "general-object-thms"))
-
-(local (include-book "hyp-fix-logic"))
-
 (include-book "bvec-ite")
 (include-book "hyp-fix")
+(include-book "split-args")
 (include-book "std/misc/two-nats-measure" :dir :system)
-
 (include-book "tools/mv-nth" :dir :system)
 (local (include-book "misc/invariants" :dir :system))
-
+(local (include-book "general-object-thms"))
+(local (include-book "hyp-fix-logic"))
 
 (verify-guards
  general-concrete-obj
@@ -31,20 +45,20 @@
         (general-number-components x))
        ((mv yrn yrd yin yid)
         (general-number-components y)))
-    (flet ((ubv-ite (c a b)
-                    (let ((res (bfr-ite-bvv-fn c a b)))
-                      (if (boolean-listp res)
-                          (v2n res)
-                        res)))
-           (sbv-ite (c a b)
-                    (let ((res (bfr-ite-bss-fn c a b)))
-                      (if (boolean-listp res)
-                          (v2i res)
-                        res))))
-      (mk-g-number (sbv-ite c xrn yrn)
-                     (ubv-ite c xrd yrd)
-                     (sbv-ite c xin yin)
-                     (ubv-ite c xid yid)))))
+    (mk-g-number (bfr-ite-bss-fn c xrn yrn)
+                 (bfr-ite-bvv-fn c xrd yrd)
+                 (bfr-ite-bss-fn c xin yin)
+                 (bfr-ite-bvv-fn c xid yid))))
+
+(defthm gobj-depends-on-of-merge-general-numbers
+  (implies (and (not (pbfr-depends-on k p c))
+                (not (gobj-depends-on k p x))
+                (not (gobj-depends-on k p y))
+                (general-numberp x)
+                (general-numberp y))
+           (not (gobj-depends-on k p (merge-general-numbers c x y))))
+  :hints(("Goal" :in-theory (enable merge-general-numbers
+                                    gobj-depends-on))))
 
 (in-theory (disable merge-general-numbers))
 
@@ -61,6 +75,16 @@
          (val (bfr-ite c av bv)))
     (mk-g-boolean val)))
 
+(defthm gobj-depends-on-of-merge-general-booleans
+  (implies (and (not (pbfr-depends-on k p c))
+                (not (gobj-depends-on k p x))
+                (not (gobj-depends-on k p y))
+                (general-booleanp x)
+                (general-booleanp y))
+           (not (gobj-depends-on k p (merge-general-booleans c x y))))
+  :hints(("Goal" :in-theory (enable merge-general-booleans
+                                    gobj-depends-on))))
+
 (in-theory (disable merge-general-booleans))
 
 
@@ -114,7 +138,13 @@
         ((eq (tag x) :g-apply)
          (if (eq (tag y) :g-apply)
              (if (equal (g-apply->fn x) (g-apply->fn y))
-                 'applies
+                 (if (equal (len (g-apply->args x))
+                            (len (g-apply->args y)))
+                     'applies
+                   (if (< (len (g-apply->args x))
+                          (len (g-apply->args y)))
+                       '<
+                     '>))
                (if (hlexorder (cdr x) (cdr y)) '< '>))
            '<))
         ((eq (tag y) :g-apply) '>)
@@ -156,6 +186,18 @@
           (bool-cond-itep-falsebr x))
     (mv t x nil)))
 
+(defthm gobj-depends-on-of-breakdown-ite-by-cond
+  (implies (not (gobj-depends-on k p x))
+           (b* (((mv test x y) (breakdown-ite-by-cond x)))
+             (and (not (pbfr-depends-on k p test))
+                  (not (gobj-depends-on k p x))
+                  (not (gobj-depends-on k p y)))))
+  :hints(("Goal" :in-theory (enable breakdown-ite-by-cond
+                                    bool-cond-itep
+                                    bool-cond-itep-truebr
+                                    bool-cond-itep-falsebr
+                                    bool-cond-itep-cond))))
+
 
 
 
@@ -179,7 +221,9 @@
                (and (not (equal x y))
                     (equal (tag x) :g-apply)
                     (equal (tag y) :g-apply)
-                    (equal (g-apply->fn x) (g-apply->fn y)))))
+                    (equal (g-apply->fn x) (g-apply->fn y))
+                    (equal (len (g-apply->args x))
+                           (len (g-apply->args y))))))
    :hints (("goal" :in-theory (enable general-booleanp general-numberp
                                       general-consp general-concrete-atom
                                       tag ite-merge-ordering)))))
@@ -194,10 +238,10 @@
 (local
  (encapsulate nil
    (local (add-bfr-pat (hyp-fix . ?)))
-   (local (in-theory (disable* acl2-count integer-abs 
+   (local (in-theory (disable* acl2-count integer-abs
                                equal-of-booleans-rewrite not
                                hyp-fix-of-hyp-fixedp
-                               
+
 ;                               bfr-eval-nonnil-forward
                                default-+-2 o<
                                default-<-1
@@ -307,7 +351,15 @@
                             true-under-hyp
                             false-under-hyp)
                          ((two-nats-measure)
-                          (maybe-merge-measure)))))))))
+                          (maybe-merge-measure)))))))
+
+   (defthm ite-merge-lists-measure-thm
+     (implies (consp x)
+              (and (o< (ite-merge-measure (car x) (car y))
+                       (ite-merge-measure x y))
+                   (o< (ite-merge-measure (cdr x) (cdr y))
+                       (ite-merge-measure x y))))
+     :hints(("Goal" :in-theory (enable ite-merge-measure acl2-count))))))
 
 
 
@@ -336,24 +388,26 @@
                    :measure (maybe-merge-measure x y)))
    (let ((ordersym (ite-merge-ordering x y)))
      (case ordersym
-       (equal (mv t x))
+       (equal (mv 'merged x))
        (booleans (mv 'merged (merge-general-booleans c x y)))
        (numbers (mv 'merged (merge-general-numbers c x y)))
        (conses (let ((hyp (bfr-and hyp (hf (bfr-ite c xhyp yhyp)))))
-                 (mv 'merged (gl-cons (ite-merge (hf c)
-                                                 (general-consp-car x)
-                                                 (general-consp-car y)
-                                                 hyp)
-                                      (ite-merge (hf c)
-                                                 (general-consp-cdr x)
-                                                 (general-consp-cdr y)
-                                                 hyp)))))
+                 (mv 'merged (gl-cons ;; gl-cons-split-ite ;;
+                              (ite-merge (hf c)
+                                         (general-consp-car x)
+                                         (general-consp-car y)
+                                         hyp)
+                              (ite-merge (hf c)
+                                         (general-consp-cdr x)
+                                         (general-consp-cdr y)
+                                         hyp)))))
        (applies (let ((hyp (bfr-and hyp (hf (bfr-ite c xhyp yhyp)))))
-                  (mv 'merged (g-apply (g-apply->fn x)
-                                       (ite-merge (hf c)
-                                                  (g-apply->args x)
-                                                  (g-apply->args y)
-                                                  hyp)))))
+                  (mv 'merged (g-apply ;; gl-fncall-split-ite ;;
+                               (g-apply->fn x)
+                               (ite-merge-lists (hf c)
+                                                (g-apply->args x)
+                                                (g-apply->args y)
+                                                hyp)))))
        (otherwise (mv ordersym nil)))))
 
 
@@ -365,7 +419,8 @@
                    :hints (("goal" :do-not-induct t
                             :in-theory '(ite-merge-measure-thm
                                          merge-rest-measure-thm
-                                         maybe-merge-measure-thm)))))
+                                         maybe-merge-measure-thm
+                                         ite-merge-lists-measure-thm)))))
    (cond ((not hyp) nil)
          ((hons-equal x y) x)
          ((th c) x)
@@ -378,20 +433,36 @@
                   (maybe-merge c first-x first-y first-x-cond first-y-cond hyp)))
               (case merge-flg
                 (merged
-                 (if (and (eq first-x-cond t)
-                          (eq first-y-cond t))
-                     first
-                   (merge-rest (hf (bfr-ite c first-x-cond first-y-cond))
-                               first c rest-x rest-y hyp)))
+                 (cond ((and (eq first-x-cond t)
+                             (eq first-y-cond t))
+                        first)
+                       ((eq first-x-cond t)
+                        (mk-g-ite (mk-g-boolean (hf (bfr-or c first-y-cond)))
+                                  first rest-y))
+                       ((eq first-y-cond t)
+                        (mk-g-ite (mk-g-boolean (hf (bfr-or (bfr-not c)
+                                                            first-x-cond)))
+                                  first rest-x))
+                       (t (merge-rest (hf (bfr-ite c first-x-cond first-y-cond))
+                                      first c rest-x rest-y hyp))))
                 (< (if (eq first-x-cond t)
                        (mk-g-ite (mk-g-boolean c) first-x y)
                      (merge-rest (bfr-and c first-x-cond)
                                  first-x c rest-x y hyp)))
                 (t ;; >
                  (if (eq first-y-cond t)
-                     (mk-g-ite (mk-g-boolean c) x first-y)
+                     (mk-g-ite (mk-g-boolean (bfr-not c)) first-y x)
                    (merge-rest (bfr-and (bfr-not c) first-y-cond)
-                               first-y c x rest-y hyp)))))))))
+                               first-y c x rest-y hyp))))))))
+
+ (defun ite-merge-lists (c x y hyp)
+   ;; (if c x y), x and y lists
+   (declare (xargs :guard (equal (len x) (len y))
+                   :measure (ite-merge-measure x y)))
+   (if (atom x)
+       nil
+     (cons (ite-merge c (car x) (car y) hyp)
+           (ite-merge-lists c (cdr x) (cdr y) hyp)))))
 
 
 (in-theory (disable ite-merge merge-rest))
@@ -402,8 +473,6 @@
 
 
 
-
-
 ;; (local
 ;;  (defthm merge-general-numbers-gobjectp
 ;;    (implies (and (gobjectp a) (gobjectp b)
@@ -420,35 +489,35 @@
             (implies (boolean-listp x)
                      (equal (bfr-eval-list x env)
                             x))))
-   
-   (local (defthm rewrite-v2i-of-boolean-list
-            (implies (and (syntaxp (not (and (consp x)
-                                             (eq (car x) 'bfr-eval-list))))
-                          (bind-free '((env . (car env))) (env))
-                          (boolean-listp x))
-                     (equal (v2i x)
-                            (v2i (bfr-eval-list x env))))
-            :hints(("Goal" :in-theory (enable bfr-eval-list-when-boolean-listp)))
-            :rule-classes ((:rewrite :backchain-limit-lst 0))))
 
-   (local (defthm rewrite-v2n-of-boolean-list
-            (implies (and (syntaxp (not (and (consp x)
-                                             (eq (car x) 'bfr-eval-list))))
-                          (bind-free '((env . (car env))) (env))
-                          (boolean-listp x))
-                     (equal (v2n x)
-                            (v2n (bfr-eval-list x env))))
-            :hints(("Goal" :in-theory (enable bfr-eval-list-when-boolean-listp)))
-            :rule-classes ((:rewrite :backchain-limit-lst 0))))
+   ;; (local (defthm rewrite-v2i-of-boolean-list
+   ;;          (implies (and (syntaxp (not (and (consp x)
+   ;;                                           (eq (car x) 'bfr-eval-list))))
+   ;;                        (bind-free '((env . (car env))) (env))
+   ;;                        (boolean-listp x))
+   ;;                   (equal (v2i x)
+   ;;                          (bfr-list->s x env)))
+   ;;          :hints(("Goal" :in-theory (enable bfr-eval-list-when-boolean-listp)))
+   ;;          :rule-classes ((:rewrite :backchain-limit-lst 0))))
 
-   (defthm bfr-eval-list-of-bfr-ite-bvv-fn-under-v2n
-     (equal (v2n (bfr-eval-list (bfr-ite-bvv-fn c a b) env))
-            (v2n (if (bfr-eval c env)
-                     (bfr-eval-list a env)
-                   (bfr-eval-list b env))))
-     :hints(("Goal" :in-theory (enable bfr-ite-bvv-fn v2n)
-             :induct t)
-            (bfr-reasoning)))
+   ;; (local (defthm rewrite-v2n-of-boolean-list
+   ;;          (implies (and (syntaxp (not (and (consp x)
+   ;;                                           (eq (car x) 'bfr-eval-list))))
+   ;;                        (bind-free '((env . (car env))) (env))
+   ;;                        (boolean-listp x))
+   ;;                   (equal (v2n x)
+   ;;                          (bfr-list->u x env)))
+   ;;          :hints(("Goal" :in-theory (enable bfr-eval-list-when-boolean-listp)))
+   ;;          :rule-classes ((:rewrite :backchain-limit-lst 0))))
+
+   ;; (defthm bfr-list->u-of-bfr-ite-bvv-fn-under
+   ;;   (equal (bfr-list->u (bfr-ite-bvv-fn c a b) env)
+   ;;          (if (bfr-eval c env)
+   ;;              (bfr-list->u a env)
+   ;;            (bfr-list->u b env)))
+   ;;   :hints(("Goal" :in-theory (enable bfr-ite-bvv-fn)
+   ;;           :induct t)
+   ;;          (bfr-reasoning)))
 
    (defthm merge-general-numbers-correct
      (implies (and (general-numberp a) (general-numberp b))
@@ -526,7 +595,7 @@
 ;;                            env)))
 ;;    :hints(("Goal" :in-theory (enable hyp-fix hyp-fixedp)))))
 
-            
+
 ;; (local
 ;;  (defthm breakdown-ite-by-cond-nonnil
 ;;    (implies (and hyp (bfr-and c hyp)
@@ -564,7 +633,8 @@
                   :hints (("goal" :do-not-induct t
                            :in-theory '(ite-merge-measure-thm
                                         merge-rest-measure-thm
-                                        maybe-merge-measure-thm)))))
+                                        maybe-merge-measure-thm
+                                        ite-merge-lists-measure-thm)))))
 
 
 
@@ -642,7 +712,9 @@
                                      true-under-hyp
                                      hyp-fixedp
                                      hyp-fix
-                                     breakdown-ite-by-cond))))))))
+                                     breakdown-ite-by-cond
+                                     (:type-prescription len)
+                                     (tau-system)))))))))
 
 
 ;; (local
@@ -688,17 +760,40 @@
    (not (equal (ite-merge-ordering x y) 'merged))
    :hints(("Goal" :in-theory (enable ite-merge-ordering)))))
 
+
+
+(local (defthm generic-geval-list-when-not-consp
+         (implies (not (consp x))
+                  (equal (generic-geval-list x env) nil))
+         :hints(("Goal" :in-theory (enable generic-geval-list)))
+         :rule-classes ((:rewrite :backchain-limit-lst 0))))
+
+(local (defthm generic-geval-list-when-len-0
+         (implies (equal (len x) 0)
+                  (equal (generic-geval-list x env) nil))
+         :hints(("Goal" :in-theory (enable generic-geval-list)))
+         :rule-classes ((:rewrite :backchain-limit-lst 0))))
+
+(local (defthm len-when-not-consp
+         (implies (not (consp x))
+                  (equal (len x) 0))
+         :rule-classes ((:rewrite :backchain-limit-lst 0))))
+
+(local (defthm len-when-consp
+         (implies (consp x)
+                  (posp (len x)))
+         :rule-classes :type-prescription))
+
 (local
  (encapsulate
    nil
    (local (in-theory (e/d* (generic-geval-g-apply-p)
-                           ((force) member-equal 
+                           ((force) member-equal
                             ite-merge merge-rest maybe-merge
                             general-number-components-ev
-                            
+
                             boolean-list-bfr-eval-list
                             mv-nth
-                            (:type-prescription len)
                             default-car default-cdr
                             hons-assoc-equal
                             (:rewrite bfr-eval-booleanp)
@@ -706,11 +801,10 @@
                             generic-geval
                             hyp-fix-of-hyp-fixedp
                             eval-concrete-gobjectp
-                            acl2-numberp-v2n
                             default-unary-/
-                            
-                            (:type-prescription v2n)
-                            (:type-prescription v2i)
+
+                            len
+
                             bfr-eval-list-consts
                             default-*-1 default-*-2
                             (:type-prescription booleanp)
@@ -737,36 +831,56 @@
    ;;            :in-theory (enable false-under-hyp true-under-hyp))))
 
    (def-ite-merge-thm ite-merge-correct-lemma
-     (ite-merge (implies (bfr-eval (double-rewrite hyp) (car env))
-                         (equal (generic-geval (ite-merge c x y hyp) env)
-                                (if (bfr-eval c (car env))
-                                    (generic-geval x env)
-                                  (generic-geval y env))))
-                :name ite-merge-correct)
-     (maybe-merge (mv-let (flg ans)
-                    (maybe-merge c x y xhyp yhyp hyp)
-                    (implies (and (equal flg 'merged)
-                                  (bfr-eval hyp (car env)))
-                             (and (implies (and (bfr-eval c (car env))
-                                                (bfr-eval xhyp (car env)))
-                                           (equal (generic-geval ans env)
-                                                  (generic-geval x env)))
-                                  (implies (and (not (bfr-eval c (car env)))
-                                                (bfr-eval yhyp (car env)))
-                                           (equal (generic-geval ans env)
-                                                  (generic-geval y env))))))
-                  :name maybe-merge-correct)
-                                
-     (merge-rest (implies (bfr-eval hyp (car env))
-                          (equal (generic-geval (merge-rest firstcond first c x y hyp) env)
-                                 (if (bfr-eval firstcond (car env))
-                                     (generic-geval first env)
-                                   (if (bfr-eval c (car env))
-                                       (generic-geval x env)
-                                     (generic-geval y env)))))
-                 :name merge-rest-correct)
-     :hints (("goal" :induct (ite-merge-ind flag firstcond first xhyp yhyp c x y hyp)
-              :do-not-induct t
+     (defthm ite-merge-correct
+       (implies (bfr-eval (double-rewrite hyp) (car env))
+                (equal (generic-geval (ite-merge c x y hyp) env)
+                       (if (bfr-eval c (car env))
+                           (generic-geval x env)
+                         (generic-geval y env))))
+       :hints ('(:expand ((ite-merge c x y hyp))))
+       :flag ite-merge)
+     (defthm ite-merge-lists-correct
+      (implies (and (bfr-eval (double-rewrite hyp) (car env))
+                    (equal (len x) (len y)))
+               (equal (generic-geval-list (ite-merge-lists c x y hyp) env)
+                      (if (bfr-eval c (car env))
+                          (generic-geval-list x env)
+                        (generic-geval-list y env))))
+       :hints ('(:expand ((ite-merge-lists c x y hyp)
+                          (generic-geval-list x env)
+                          (generic-geval-list y env)
+                          (generic-geval-list nil env)
+                          (:free (a b) (generic-geval-list (cons a b) env)))))
+      :flag ite-merge-lists)
+     (defthm maybe-merge-correct
+       (mv-let (flg ans)
+         (maybe-merge c x y xhyp yhyp hyp)
+         (implies (and (equal flg 'merged)
+                       (bfr-eval hyp (car env)))
+                  (and (implies (and (bfr-eval c (car env))
+                                     (bfr-eval xhyp (car env)))
+                                (equal (generic-geval ans env)
+                                       (generic-geval x env)))
+                       (implies (and (not (bfr-eval c (car env)))
+                                     (bfr-eval yhyp (car env)))
+                                (equal (generic-geval ans env)
+                                       (generic-geval y env))))))
+       :hints ('(:expand ((maybe-merge c x y xhyp yhyp hyp)))
+               (and stable-under-simplificationp
+                    '(:in-theory (enable generic-geval))))
+       :flag maybe-merge)
+
+     (defthm merge-rest-correct
+       (implies (bfr-eval hyp (car env))
+                (equal (generic-geval (merge-rest firstcond first c x y hyp) env)
+                       (if (bfr-eval firstcond (car env))
+                           (generic-geval first env)
+                         (if (bfr-eval c (car env))
+                             (generic-geval x env)
+                           (generic-geval y env)))))
+       :hints ('(:expand ((merge-rest firstcond first c x y hyp))))
+       :flag merge-rest)
+     :hints (("goal" :do-not-induct t
               :in-theory (set-difference-theories
                           (list* '(:induction ite-merge-ind)
                                  '(:rewrite ite-merge-ind-equivalences)
@@ -778,10 +892,10 @@
              ;;            ("Subgoal *1/13" :by nil)
              ;;            ("Subgoal *1/12" :by nil)
              ;;            ("Subgoal *1/11" :by nil)
-             (and ;;(subgoal-of "Subgoal *1/" id)
-              stable-under-simplificationp
-              (flag::expand-calls-computed-hint
-               clause '(ite-merge merge-rest maybe-merge)))
+             ;; (and ;;(subgoal-of "Subgoal *1/" id)
+             ;;  stable-under-simplificationp
+             ;;  (flag::expand-calls-computed-hint
+             ;;   clause '(ite-merge merge-rest maybe-merge)))
              (and ;;(subgoal-of "Subgoal *1/" id)
               stable-under-simplificationp
               (or (cw "enabling~%")
@@ -798,14 +912,60 @@
                                      ;; ite-merge-guard merge-rest-guard
                                      ;; maybe-merge-guard
                                      hyp-fix hyp-fixedp)
-                                    ()))))))))
+                                    ()))))))
 
+   (local (Defthm gobj-list-depends-on-nil
+            (not (gobj-list-depends-on k p nil))
+            :hints(("Goal" :in-theory (enable gobj-list-depends-on)))))
+
+   (def-ite-merge-thm gobj-depends-on-of-ite-merge-lemma
+     (defthm gobj-depends-on-of-ite-merge
+       (implies (and (not (pbfr-depends-on k p c))
+                     (not (gobj-depends-on k p x))
+                     (not (gobj-depends-on k p y)))
+                (not (gobj-depends-on k p (ite-merge c x y hyp))))
+       :hints ('(:expand ((ite-merge c x y hyp)
+                          (ite-merge c x y nil)
+                          (ite-merge c x x hyp))))
+       :flag ite-merge)
+     (defthm gobj-depends-on-of-ite-merge-lists
+       (implies (and (not (pbfr-depends-on k p c))
+                     (not (gobj-list-depends-on k p x))
+                     (not (gobj-list-depends-on k p y)))
+                (not (gobj-list-depends-on k p (ite-merge-lists c x y hyp))))
+       :hints ('(:expand ((ite-merge-lists c x y hyp))))
+      :flag ite-merge-lists)
+     (defthm gobj-depends-on-of-maybe-merge
+       (mv-let (flg ans)
+         (maybe-merge c x y xhyp yhyp hyp)
+         (declare (ignore flg))
+         (implies (and (not (pbfr-depends-on k p c))
+                       (not (gobj-depends-on k p x))
+                       (not (gobj-depends-on k p y)))
+                  (not (gobj-depends-on k p ans))))
+       :hints ('(:expand ((maybe-merge c x y xhyp yhyp hyp)
+                          (maybe-merge c x x xhyp yhyp hyp)))
+               (and stable-under-simplificationp
+                    '(:in-theory (enable generic-geval))))
+       :flag maybe-merge)
+
+     (defthm gobj-depends-on-of-merge-rest
+       (implies (and (not (pbfr-depends-on k p firstcond))
+                     (not (gobj-depends-on k p first))
+                     (not (pbfr-depends-on k p c))
+                     (not (gobj-depends-on k p x))
+                     (not (gobj-depends-on k p y)))
+                (not (gobj-depends-on k p (merge-rest firstcond first c x y hyp))))
+       :hints ('(:expand ((merge-rest firstcond first c x y hyp)
+                          (merge-rest firstcond first c x y nil))))
+       :flag merge-rest)
+     :hints (("goal" :do-not-induct t)))))
 
 
 (verify-guards ite-merge
                :hints (("Goal" :in-theory (e/d** ((:ruleset minimal-rules))))
                        (and stable-under-simplificationp
-                            '(:in-theory 
+                            '(:in-theory
                               (e/d** ((:ruleset minimal-rules)
                                       ite-merge-invariants))))
                        (and stable-under-simplificationp
@@ -865,3 +1025,10 @@
                     (generic-geval y env))))
   :hints(("Goal" :in-theory (e/d (gobj-ite-merge)))))
 
+(defthm gobj-depends-on-of-gobj-ite-merge
+  (implies (and (not (pbfr-depends-on k p c))
+                (not (gobj-depends-on k p x))
+                (not (gobj-depends-on k p y)))
+           (not (gobj-depends-on k p (gobj-ite-merge c x y hyp))))
+  :hints(("Goal" :in-theory (enable gobj-ite-merge))))
+
