@@ -29,14 +29,14 @@
 
 (local (include-book "arithmetic/top" :dir :system))
 
-(in-theory (disable* default-car default-cdr ;; blp-implies-t
-                     default-+-2 default-+-1 default-<-2 default-<-1
-                     (:ruleset canonicalize-to-q-ite)
-                     equal-by-eval-bdds
-                     ))
+(local (in-theory (disable* default-car default-cdr ;; blp-implies-t
+                            default-+-2 default-+-1 default-<-2 default-<-1
+                            (:ruleset canonicalize-to-q-ite)
+                            equal-by-eval-bdds
+                            )))
 
-(in-theory (enable eval-bdd eval-bdd-list ubddp ubdd-listp
-                   q-compose q-compose-list))
+(local (in-theory (enable eval-bdd eval-bdd-list ubddp ubdd-listp
+                          q-compose q-compose-list)))
 
 (make-event
 
@@ -147,8 +147,7 @@
 ;; more.  2) only holds for variable assignments of length NVARS.
 (defn q-param (x nvars)
   (cond ((zp (nfix nvars)) nil)
-        ((atom x) (if x (qv-list 0 1 nvars)
-                    (make-list-ac nvars nil nil)))
+        ((atom x) (qv-list 0 1 nvars))
         ((not (car x))
          (cons nil (q-param (cdr x) (1- nvars))))
         ((not (cdr x))
@@ -709,7 +708,7 @@
 (memoize 'from-param-space :condition '(or (consp p) (consp y)))
 
 (defn to-param-space (p y)
-  (cond ((atom p) (if p y nil))
+  (cond ((atom p) y)
         ((atom y) y)
         ((eq (car p) nil)
          (to-param-space (cdr p) (cdr y)))
@@ -734,8 +733,8 @@
 
 (defun param-env (p env)
   (declare (xargs :guard t))
-  (cond ((atom env) nil)
-        ((atom p) env)
+  (cond ((atom p) env)
+        ((atom env) nil)
         ((eq (car p) nil) (param-env (cdr p) (cdr env)))
         ((eq (cdr p) nil) (param-env (car p) (cdr env)))
         ((car env) (cons t (param-env (car p) (cdr env))))
@@ -759,10 +758,35 @@
              (cons car (unparam-env (if car (car p) (cdr p))
                                     cdr))))))
 
+
+(encapsulate
+  nil
+  (local (defun ind (x p env)
+           (if (consp x)
+               (if (consp p)
+                   (if (car p)
+                       (if (cdr p)
+                           (if (car env)
+                               (ind (car x) (car p) (cdr env))
+                             (ind (cdr x) (cdr p) (cdr env)))
+                         (ind x (car p) env))
+                     (ind x (cdr p) env))
+                 x)
+             env)))
+
+  (defthm eval-param-env-of-unparam-env
+    (equal (eval-bdd x (param-env p (unparam-env p env)))
+           (eval-bdd x env))
+    :hints (("goal" :induct (ind x p env)
+             :in-theory (enable default-car default-cdr))
+            (and stable-under-simplificationp
+                 '(:expand ((:free (env) (eval-bdd x env))))))))
+
 (defthm eval-with-unparam-env
   (implies (and p (ubddp p))
            (eval-bdd p (unparam-env p env))))
 
+
 (defun unparam-env-ind (x p env)
   (cond ((atom p) (list x env))
         ((eq (car p) nil) (unparam-env-ind (cdr x) (cdr p) env))
@@ -772,15 +796,20 @@
                             (cdr env)))))
 
 (defthmd unparam-env-to-param-space
-  (implies (and p (ubddp p))
-           (equal (eval-bdd (to-param-space p x) env)
-                  (eval-bdd x (unparam-env p env))))
+  (equal (eval-bdd (to-param-space p x) env)
+         (eval-bdd x (unparam-env p env)))
   :hints (("goal" :induct (unparam-env-ind x p env)
            :expand ((:free (env) (eval-bdd x env))
                     (:free (env) (unparam-env p env)))
            :in-theory (enable default-car default-cdr))))
 
-
+(defthm unparam-env-of-param-env
+  (implies (eval-bdd p env)
+           (equal (eval-bdd x (unparam-env p (param-env p env)))
+                  (eval-bdd x env)))
+  :hints (("goal" :use ((:instance param-env-to-param-space)
+                        (:instance unparam-env-to-param-space
+                         (env (param-env p env)))))))
 
 (defn to-param-space-list (p list)
   (if (atom list)
@@ -912,6 +941,47 @@
   :hints (("goal" :induct (to-param-space p x)
            :in-theory (e/d (q-and) (ubddp)))))
 
+(defthm eval-bdd-of-qcons
+  (equal (eval-bdd (qcons x y) env)
+         (if (car env)
+             (eval-bdd x (cdr env))
+           (eval-bdd y (cdr env)))))
+
+(defun to-from-ind (p x env)
+  (if (consp p)
+      (cond ((eq (car p) nil)
+             (and (not (car env))
+                  (to-from-ind (cdr p) (cdr x) (cdr env))))
+            ((eq (cdr p) nil)
+             (and (car env)
+                  (to-from-ind (car p) (car x) (cdr env))))
+            (t (list (to-from-ind (car p) (car x) (cdr env))
+                     (to-from-ind (cdr p) (cdr x) (cdr env)))))
+    (list p x env)))
+    
+
+(defthm from-param-space-of-qcons
+  (implies (and (car p) (cdr p))
+           (equal (from-param-space p (qcons x y))
+                  (if (atom (qcons x y))
+                      (and x p)
+                    (qcons (from-param-space (car p) x)
+                           (from-param-space (cdr p) y)))))
+  :otf-flg t)
+
+(defthm consp-qcons
+  (equal (consp (qcons x y))
+         (not (and (booleanp x)
+                   (equal x y)))))
+
+(defthm to-from-param-space-eval
+  (equal (eval-bdd (from-param-space p (to-param-space p x)) env)
+         (and (eval-bdd p env)
+              (eval-bdd x env)))
+  :hints (("goal" :induct (to-from-ind p x env)
+           :in-theory (disable qcons))))
+
+
 (defthm to-param-space-self
   (implies (and (ubddp p) p)
            (equal (to-param-space p p) t))
@@ -924,3 +994,23 @@
                   x))
   :hints (("goal" :induct (from-param-space p x)
            :in-theory (e/d (q-and) (ubddp)))))
+
+
+
+(defun param-env-ind (x p env)
+  (cond ((atom p) (list x env))
+        ((eq (car p) nil) (param-env-ind x (cdr p) (cdr env)))
+        ((eq (cdr p) nil) (param-env-ind x (car p) (cdr env)))
+        ((car env) (cons t (param-env-ind (car x) (car p) (cdr env))))
+        (t       (cons nil (param-env-ind (cdr x) (cdr p) (cdr env))))))
+
+(defthm eval-of-from-param-space
+  (implies (eval-bdd p env)
+           (equal (eval-bdd (from-param-space p x)
+                            env)
+                  (eval-bdd x (param-env p env))))
+  :hints (("goal" :induct (param-env-ind x p env)
+           :expand ((from-param-space p x)
+                    (unparam-env p env)
+                    (:free (env) (eval-bdd x env))
+                    (eval-bdd p env)))))
