@@ -22,7 +22,7 @@
 (include-book "vcd-stub")
 (include-book "oslib/date" :dir :system)
 (include-book "centaur/misc/tshell" :dir :system)
-(include-book "centaur/aig/three-four" :dir :system)
+(include-book "centaur/aig/faig-constructors" :dir :system)
 (include-book "centaur/misc/hons-extra" :dir :system)
 (include-book "centaur/vl/util/print" :dir :system)
 (include-book "centaur/vl/toe/toe-emodwire" :dir :system)
@@ -71,7 +71,7 @@
 ; The VALUE bound to each name may be:
 ;
 ;    - Concrete FAIGs
-;          i.e., *4f*, *4t*, *4x*, or *4z*
+;          i.e., (faig-f), (faig-t), (faig-x), or (faig-z)
 ;
 ;    - Concrete S-Expression Results
 ;          i.e., *4vf*, *4vt*, *4vx*, or *4vz*   ('F, 'T, 'X, 'Z)
@@ -147,7 +147,7 @@
      (implies (string-listp x)
               (iff (first (last x))
                    (consp x)))
-     :hints(("Goal" :in-theory (disable (force)))))
+     :hints(("Goal" :in-theory (e/d (last) ((force))))))
 
    (defthm stringp-of-car-when-string-listp
      (implies (string-listp x)
@@ -306,10 +306,10 @@
   (defund vcd-value-p (x)
     (declare (xargs :guard t))
     (if (consp x)
-        (or (equal x acl2::*4f*)
-            (equal x acl2::*4t*)
-            (equal x acl2::*4x*)
-            (equal x acl2::*4z*))
+        (or (equal x (acl2::faig-f))
+            (equal x (acl2::faig-t))
+            (equal x (acl2::faig-x))
+            (equal x (acl2::faig-z)))
       (or (eq x nil)
           (eq x 'acl2::f)
           (eq x t)
@@ -323,10 +323,10 @@
           (er hard? 'vcd-value->char "Not a vcd-value-p: ~x0." x)
           #\x)
          ((when (consp x))
-          (cond ((equal x acl2::*4f*) #\0)
-                ((equal x acl2::*4t*) #\1)
-                ((equal x acl2::*4z*) #\z)
-                (t                    #\x))))
+          (cond ((equal x (acl2::faig-f)) #\0)
+                ((equal x (acl2::faig-t)) #\1)
+                ((equal x (acl2::faig-z)) #\z)
+                (t                        #\x))))
       (case x
         ((nil acl2::f) #\0)
         ((t)           #\1)
@@ -341,10 +341,10 @@
      (assert! (equal (vcd-value->char 'acl2::x) #\x))
      (assert! (equal (vcd-value->char 'acl2::z) #\z))
      (assert! (equal (vcd-value->char 'acl2::u) #\z))
-     (assert! (equal (vcd-value->char acl2::*4f*) #\0))
-     (assert! (equal (vcd-value->char acl2::*4t*) #\1))
-     (assert! (equal (vcd-value->char acl2::*4x*) #\x))
-     (assert! (equal (vcd-value->char acl2::*4z*) #\z)))))
+     (assert! (equal (vcd-value->char (acl2::faig-f)) #\0))
+     (assert! (equal (vcd-value->char (acl2::faig-t)) #\1))
+     (assert! (equal (vcd-value->char (acl2::faig-x)) #\x))
+     (assert! (equal (vcd-value->char (acl2::faig-z)) #\z)))))
 
 
 (defsection vcd-valuelist-p
@@ -1826,4 +1826,4 @@
 (progn!
  (set-raw-mode t)
  (defun vcd-dump-fn (filename snapshots viewer emap state)
-   (vcd-dump-fn-real filename snapshots viewer emap state)))
\ No newline at end of file
+   (vcd-dump-fn-real filename snapshots viewer emap state)))
