@@ -376,10 +376,10 @@ allowed.</p>"
     ;; Just to make sure we keep ths up to date if we ever change
     ;; vl-modinst-allexprs
     (let ((ret (vl-modinst-exprs-for-implicit-wires x)))
-      (set-equivp (append (mv-nth 0 ret)
+      (set-equiv (append (mv-nth 0 ret)
                              (mv-nth 1 ret))
                      (vl-modinst-allexprs x)))
-    :hints(("Goal" :in-theory (enable set-equivp vl-modinst-allexprs)))))
+    :hints(("Goal" :in-theory (enable set-equiv vl-modinst-allexprs)))))
 
 
 
@@ -413,10 +413,10 @@ instance (its range, delay) where implicit wires aren't allowed.</p>"
     ;; Just to make sure we keep ths up to date if we ever change
     ;; vl-gateinst-allexprs
     (let ((ret (vl-gateinst-exprs-for-implicit-wires x)))
-      (set-equivp (append (mv-nth 0 ret)
+      (set-equiv (append (mv-nth 0 ret)
                              (mv-nth 1 ret))
                      (vl-gateinst-allexprs x)))
-    :hints(("Goal" :in-theory (enable set-equivp vl-gateinst-allexprs)))))
+    :hints(("Goal" :in-theory (enable set-equiv vl-gateinst-allexprs)))))
 
 
 
