@@ -96,12 +96,11 @@ optimization.</p>")))
          (implies (force (,rec x))
                   (vl-exprlist-p (,collect x))))
 
+       (never-memoize ,collect-exec)
        (defttag vl-optimize)
-       (progn!
-        (set-raw-mode t)
-        (setf (gethash ',collect-exec ACL2::*never-profile-ht*) t)
-        (defun ,collect (x)
-          (nreverse (,collect-exec x nil))))
+       (progn! (set-raw-mode t)
+               (defun ,collect (x)
+                 (nreverse (,collect-exec x nil))))
        (defttag nil))))
 
 
@@ -367,16 +366,14 @@ list-rec-s "), as described in @(see allexprs).</p>")))
   :type vl-delayoreventcontrol
   :exec-body
   (case (tag x)
-    (:vl-delaycontrol (vl-delaycontrol-allexprs-exec x acc))
-    (:vl-eventcontrol (vl-eventcontrol-allexprs-exec x acc))
+    (:vl-delaycontrol        (vl-delaycontrol-allexprs-exec x acc))
+    (:vl-eventcontrol        (vl-eventcontrol-allexprs-exec x acc))
     (:vl-repeat-eventcontrol (vl-repeateventcontrol-allexprs-exec x acc))
-    (otherwise
-     (prog2$ (er hard 'vl-delayoreventcontrol-allexprs-exec "Provably impossible.")
-             acc)))
+    (otherwise               (or (impossible) acc)))
   :body
   (case (tag x)
-    (:vl-delaycontrol (vl-delaycontrol-allexprs x))
-    (:vl-eventcontrol (vl-eventcontrol-allexprs x))
+    (:vl-delaycontrol        (vl-delaycontrol-allexprs x))
+    (:vl-eventcontrol        (vl-eventcontrol-allexprs x))
     (:vl-repeat-eventcontrol (vl-repeateventcontrol-allexprs x))))
 
 (def-vl-allexprs
@@ -430,9 +427,7 @@ list-rec-s "), as described in @(see allexprs).</p>")))
     (:vl-enablestmt       (vl-enablestmt-allexprs-exec x acc))
     (:vl-disablestmt      (vl-disablestmt-allexprs-exec x acc))
     (:vl-eventtriggerstmt (vl-eventtriggerstmt-allexprs-exec x acc))
-    (otherwise
-     (prog2$ (er hard 'vl-atomicstmt-allexprs-exec "Provably impossible.")
-             acc)))
+    (otherwise            (or (impossible) acc)))
   :body
   (case (tag x)
     (:vl-nullstmt         nil)
@@ -450,9 +445,7 @@ list-rec-s "), as described in @(see allexprs).</p>")))
     (:vl-vardecl   (vl-vardecl-allexprs-exec x acc))
     (:vl-eventdecl (vl-eventdecl-allexprs-exec x acc))
     (:vl-paramdecl (vl-paramdecl-allexprs-exec x acc))
-    (otherwise
-     (prog2$ (er hard 'vl-blockitem-allexprs-exec "Provably impossible.")
-             acc)))
+    (otherwise     (or (impossible) acc)))
   :body
   (case (tag x)
     (:vl-regdecl   (vl-regdecl-allexprs x))
@@ -550,15 +543,14 @@ the collection.  Under the hood, we also use @('nreverse') optimization.</p>"
                  :hints(("Goal" :in-theory (enable vl-stmt-allexprs
                                                    vl-stmtlist-allexprs))))
 
+  (never-memoize vl-stmt-allexprs-exec)
+  (never-memoize vl-stmtlist-allexprs-exec)
   (defttag vl-optimize)
-  (progn!
-   (set-raw-mode t)
-   (setf (gethash 'vl-stmt-allexprs-exec ACL2::*never-profile-ht*) t)
-   (setf (gethash 'vl-stmtlist-allexprs-exec ACL2::*never-profile-ht*) t)
-   (defun vl-stmt-allexprs (x)
-     (nreverse (vl-stmt-allexprs-exec x nil)))
-   (defun vl-stmtlist-allexprs (x)
-     (nreverse (vl-stmtlist-allexprs-exec x nil))))
+  (progn! (set-raw-mode t)
+          (defun vl-stmt-allexprs (x)
+            (nreverse (vl-stmt-allexprs-exec x nil)))
+          (defun vl-stmtlist-allexprs (x)
+            (nreverse (vl-stmtlist-allexprs-exec x nil))))
   (defttag nil)
 
   (defthm-vl-flag-stmt-p lemma
