@@ -22,6 +22,17 @@
 (include-book "../parsetree")
 (local (include-book "../util/arithmetic"))
 
+(make-event
+
+; Disabling waterfall parallelism because this book allegedly uses memoization
+; while performing its proofs.  
+
+ (if (and (ACL2::hons-enabledp state) 
+          (f-get-global 'ACL2::parallel-execution-enabled state)) 
+     (er-progn (set-waterfall-parallelism nil)
+               (value '(value-triple nil)))
+   (value '(value-triple nil))))
+
 (defxdoc expr-tools
   :parents (mlib)
   :short "Basic functions for working with expressions.")
@@ -497,14 +508,13 @@ expression list."
                                                    vl-exprlist-names-exec))))
 
   (defttag vl-optimize)
-  (progn!
-   (set-raw-mode t)
-   (setf (gethash 'vl-expr-names-exec ACL2::*never-profile-ht*) t)
-   (setf (gethash 'vl-exprlist-names-exec ACL2::*never-profile-ht*) t)
-   (defun vl-expr-names (x)
-     (nreverse (vl-expr-names-exec x nil)))
-   (defun vl-exprlist-names (x)
-     (nreverse (vl-exprlist-names-exec x nil))))
+  (never-memoize vl-expr-names-exec)
+  (never-memoize vl-exprlist-names-exec)
+  (progn! (set-raw-mode t)
+          (defun vl-expr-names (x)
+            (nreverse (vl-expr-names-exec x nil)))
+          (defun vl-exprlist-names (x)
+            (nreverse (vl-exprlist-names-exec x nil))))
   (defttag nil)
 
   (local (defthm lemma
@@ -566,8 +576,8 @@ expression list."
                     (subsetp-equal (vl-expr-names a)
                                    (vl-exprlist-names y)))))
 
-  (defcong set-equivp set-equivp (vl-exprlist-names x) 1
-    :hints(("Goal" :in-theory (enable set-equivp)))))
+  (defcong set-equiv set-equiv (vl-exprlist-names x) 1
+    :hints(("Goal" :in-theory (enable set-equiv)))))
 
 
 
@@ -625,14 +635,13 @@ may contain any @(see vl-op-p), including even odd such as @(':vl-syscall') or
           :exec (reverse (vl-fast-exprlist-ops x nil)))))
 
   (defttag vl-optimize)
-  (progn!
-   (set-raw-mode t)
-   (setf (gethash 'vl-fast-expr-ops ACL2::*never-profile-ht*) t)
-   (setf (gethash 'vl-fast-exprlist-ops ACL2::*never-profile-ht*) t)
-   (defun vl-expr-ops (x)
-     (nreverse (vl-fast-expr-ops x nil)))
-   (defun vl-exprlist-ops (x)
-     (nreverse (vl-fast-exprlist-ops x nil))))
+  (never-memoize vl-fast-expr-ops)
+  (never-memoize vl-fast-exprlist-ops)
+  (progn! (set-raw-mode t)
+          (defun vl-expr-ops (x)
+            (nreverse (vl-fast-expr-ops x nil)))
+          (defun vl-exprlist-ops (x)
+            (nreverse (vl-fast-exprlist-ops x nil))))
   (defttag nil)
 
   (defthm true-listp-of-vl-expr-ops
@@ -955,10 +964,10 @@ usually think of as atoms.</p>
           :exec (reverse (vl-fast-exprlist-atoms x nil)))))
 
   (defttag vl-optimize)
+  (never-memoize vl-fast-expr-atoms)
+  (never-memoize vl-fast-exprlist-atoms)
   (progn!
    (set-raw-mode t)
-   (setf (gethash 'vl-fast-expr-atoms ACL2::*never-profile-ht*) t)
-   (setf (gethash 'vl-fast-exprlist-atoms ACL2::*never-profile-ht*) t)
    (defun vl-expr-atoms (x)
      (nreverse (vl-fast-expr-atoms x nil)))
    (defun vl-exprlist-atoms (x)
@@ -1185,3 +1194,21 @@ expression, and return them as a flat list of expressions."
     :hints(("Goal" :expand ((vl-exprlist-selects x)
                             (vl-expr-selects x))))))
 
+
+(define vl-bitlist-from-nat ((x natp)
+                             (width natp))
+  :returns (bits vl-bitlist-p)
+  :parents (expr-tools)
+  :short "Turn a natural number into a vl-bitlist-p of the given width."
+  (b* (((when (zp width)) nil)
+       (width (1- width))
+       (bit (if (logbitp width x)
+                :vl-1val
+              :vl-0val)))
+    (cons bit (vl-bitlist-from-nat x width)))
+
+  ///
+
+  (defthm len-of-vl-bitlist-from-nat
+    (equal (len (vl-bitlist-from-nat x width))
+           (nfix width))))
