@@ -250,7 +250,7 @@ are to blame.</li>
                              remove-equal
                              promote-member-equal-to-membership
                              true-listp
-                             acl2::subsetp-equal-member
+                             acl2::subsetp-member
                              acl2::true-listp-member-equal)))
 
   (defconst *vl-bad-submodule-message*
@@ -332,13 +332,12 @@ are to blame.</li>
   (verify-guards vl-apply-blame-alist)
 
   (defttag vl-optimize)
-  (progn!
-   (set-raw-mode t)
-   (setf (gethash 'vl-apply-blame-alist-exec acl2::*never-profile-ht*) t)
-   (defun vl-apply-blame-list (mods alist)
-     (mv-let (survivors victims)
-       (vl-apply-blame-alist-exec mods alist nil nil)
-       (mv (nreverse survivors) (nreverse victims)))))
+  (never-memoize vl-apply-blame-alist-exec)
+  (progn! (set-raw-mode t)
+          (defun vl-apply-blame-list (mods alist)
+            (mv-let (survivors victims)
+              (vl-apply-blame-alist-exec mods alist nil nil)
+              (mv (nreverse survivors) (nreverse victims)))))
   (defttag nil)
 
   (defthm vl-modulelist-p-of-vl-apply-blame-alist-0
@@ -398,7 +397,7 @@ are to blame.</li>
                                  remove-equal
                                  promote-member-equal-to-membership
                                  true-listp
-                                 acl2::subsetp-equal-member
+                                 acl2::subsetp-member
                                  acl2::true-listp-member-equal)
              :do-not '(generalize fertilize)))))
 
@@ -408,7 +407,7 @@ are to blame.</li>
     ;; this is a good rule.
     (implies
      (force (no-duplicatesp-equal (vl-modulelist->names mods)))
-     (set-equivp
+     (set-equiv
       (vl-modulelist->names (mv-nth 1 (vl-apply-blame-alist mods alist)))
       (set-difference-equal
        (vl-modulelist->names mods)
@@ -480,7 +479,7 @@ modules, and separate them from the modules that are okay.</p>
     ;; names to be in terms of the survivors' names.  It's not clear whether
     ;; this is a good rule.
     (implies (force (no-duplicatesp-equal (vl-modulelist->names mods)))
-             (set-equivp
+             (set-equiv
               (vl-modulelist->names (mv-nth 1 (vl-remove-bad-modules names mods)))
               (set-difference-equal
                (vl-modulelist->names mods)
@@ -572,7 +571,7 @@ same order as they occur in @('x').</p>"
     ;; names to be in terms of the survivors' names.  It's not clear whether
     ;; this is a good rule.
     (implies (force (no-duplicatesp-equal (vl-modulelist->names mods)))
-             (set-equivp
+             (set-equiv
               (vl-modulelist->names (mv-nth 1 (vl-propagate-errors mods)))
               (set-difference-equal
                (vl-modulelist->names mods)
