@@ -217,7 +217,7 @@ compound-stmt begin formed.</p>"
   ;; I don't bother with make-honsed-vl-ifstmt, but we could add that easily
   ;; enough, if desired...
   `(progn
-     ,(cutil::da-make-maker-fn name fields)
+     ,(cutil::da-make-maker-fn name fields nil)
      ,(cutil::da-make-maker name fields)
      ,(cutil::da-make-changer-fn name fields)
      ,(cutil::da-make-changer name fields)
