@@ -243,7 +243,7 @@ interpretation by the EMOD hardware simulator, and represented the constant
 true and false functions.  These wires no longer have a special meaning in
 ESIM, but throughout VL our notion of emodwires still assumes that T and F
 stand for constant true and false, and, e.g., we still rely on this in @(see
-make-esim).  We might eventually get away from this by using a transform
+e-conversion).  We might eventually get away from this by using a transform
 analagous to @(see weirdint-elim) to introduce T/F wires and eliminate
 constants.</p>
 
@@ -277,11 +277,11 @@ replacements.  This should not be too confusing since, e.g., in Verilog
                          (stringp name))
                     (equal (equal x name)
                            (and (stringp x)
-                                (equal (coerce x 'list) (coerce name 'list)))))
+                                (equal (explode x) (explode name)))))
            :hints(("Goal"
-                   :in-theory (disable acl2::coerce-inverse-2)
-                   :use ((:instance acl2::coerce-inverse-2 (acl2::x x))
-                         (:instance acl2::coerce-inverse-2 (acl2::x name)))))))
+                   :in-theory (disable str::implode-of-explode)
+                   :use ((:instance str::implode-of-explode (str::x x))
+                         (:instance str::implode-of-explode (str::x name)))))))
 
   (local (defthm open-equal-len
            (implies (syntaxp (quotep n))
@@ -304,7 +304,8 @@ replacements.  This should not be too confusing since, e.g., in Verilog
   (local (in-theory (enable len)))
 
   (definlined vl-plain-wire-name (name)
-    (declare (xargs :guard (stringp name)))
+    (declare (xargs :guard (stringp name)
+                    :guard-hints(("Goal" :in-theory (disable str::explode-under-iff)))))
     (mbe :logic
          (cond ((equal name "T")
                 (make-vl-emodwire :basename "T" :index 0))
@@ -1312,7 +1313,8 @@ some width and value.  We return a <i>width</i>-long list of symbols
   (defthm true-listp-of-vl-msb-constint-bitlist-1
     (implies (true-listp warnings)
              (true-listp (mv-nth 1 (vl-msb-constint-bitlist x warnings))))
-    :rule-classes :type-prescription)
+    :rule-classes :type-prescription
+    :hints(("Goal" :in-theory (disable (force)))))
 
   (local (defthm vl-emodwirelist-p-of-make-list-ac
            (implies (and (vl-emodwirelist-p ac)
