@@ -26,7 +26,7 @@
 
 (include-book "cutil/defrule" :dir :system)
 (include-book "centaur/misc/equal-sets" :dir :system)
-(include-book "finite-set-theory/osets/sets" :dir :system)
+(include-book "std/osets/top" :dir :system)
 (include-book "std/lists/list-fix" :dir :system)
 (include-book "std/lists/take" :dir :system)
 (include-book "defsort/duplicated-members" :dir :system)
