@@ -27,7 +27,8 @@
 
 (local (defthm hons-assoc-equal-under-iff
          (iff (hons-assoc-equal a x)
-              (member-equal a (alist-keys x)))))
+              (member-equal a (alist-keys x)))
+         :hints(("Goal" :in-theory (enable hons-assoc-equal)))))
 
 
 
@@ -372,7 +373,7 @@ toposort-aux)."
            (implies (and (mv-nth 0 (toposort-aux queue seen graph))
                          (subsetp-equal queue (alist-keys graph)))
                     (subsetp-equal queue (eto (mv-nth 1 (toposort-aux queue seen graph)))))
-           :hints((acl2::witness :ruleset (acl2::subsetp-equal-witnessing)))))
+           :hints((acl2::witness :ruleset (acl2::subsetp-witnessing)))))
 
   (defthm extract-topological-order-includes-queue
     (implies (and (mv-nth 0 (toposort-aux queue seen graph))
@@ -585,7 +586,7 @@ strategy is the easiest way to topologically sort your data.</p>"
   (local (defthm l2
            (implies (and (consp x)
                          (cons-listp x))
-                    (set-equivp (cons (caar x) (alist-keys (cdr x)))
+                    (set-equiv (cons (caar x) (alist-keys (cdr x)))
                                 (alist-keys x)))))
 
   (local (make-event
@@ -604,7 +605,7 @@ strategy is the easiest way to topologically sort your data.</p>"
                                         (stop (caar ,seen))
                                         (acc  (list (caar ,seen)))
                                         (full-seen ,seen))))
-                      (acl2::witness :ruleset (acl2::subsetp-equal-witnessing)))))))
+                      (acl2::witness :ruleset (acl2::subsetp-witnessing)))))))
 
 
   (defthm subsetp-equal-of-toposort
@@ -629,9 +630,9 @@ strategy is the easiest way to topologically sort your data.</p>"
 
   (defthm toposort-set-equiv
     (implies (mv-nth 0 (toposort graph))
-             (set-equivp (mv-nth 1 (toposort graph))
+             (set-equiv (mv-nth 1 (toposort graph))
                          (alist-keys graph)))
-    :hints(("Goal" :in-theory (e/d (set-equivp)
+    :hints(("Goal" :in-theory (e/d (set-equiv)
                                    (toposort)))))
 
   (defthm no-duplicatesp-equal-of-toposort
@@ -651,7 +652,7 @@ strategy is the easiest way to topologically sort your data.</p>"
 ;; Some basic unit tests.  It's okay if these particular orders change, as long
 ;; as they're still valid.
 
-   (include-book "make-event/assert" :dir :system)
+   (include-book "misc/assert" :dir :system)
 
    (assert!
     (mv-let (successp order)
