@@ -52,6 +52,20 @@ reasoning about @('car') in general.</p>"
     :rule-classes :forward-chaining
     :hints(("Goal" :in-theory (enable tag)))))
 
+(deftheory defaggregate-basic-theory
+  (union-theories
+   '(tag
+     car-cons
+     cdr-cons
+     alistp
+     assoc-equal
+     hons
+     booleanp
+     booleanp-compound-recognizer
+     tag)
+   (theory 'minimal-theory)))
+
+
 (program)
 
 ; NAME GENERATION.  We introduce some functions to generate the names of
@@ -270,7 +284,24 @@ reasoning about @('car') in general.</p>"
   ;; inline accessors.
   (let ((foo (da-constructor-name basename)))
     `(defund ,foo ,fields
-       (declare (xargs :guard ,guard))
+       (declare (xargs :guard ,guard
+                       :guard-hints
+                       (("Goal" :in-theory (theory 'minimal-theory))
+                        (and stable-under-simplificationp
+                             ;; I hadn't expected to need to do this, because
+                             ;; the constructor is just consing something
+                             ;; together, so how could it have guard
+                             ;; obligations?
+                             ;;
+                             ;; But it turns out that it CAN have other guard
+                             ;; obligations, since the ,guard above can be
+                             ;; arbitrarily complicated.  So, we will rely on
+                             ;; the user to provide a theory that can satisfy
+                             ;; these obligations.
+                             ;;
+                             ;; This looks like it does nothing, but really it
+                             ;; "undoes" the in-theory event above.
+                             '(:in-theory (enable ))))))
        ,(da-pack-fields honsp legiblep tag fields))))
 
 (defun da-make-honsed-constructor-raw (basename tag fields guard legiblep)
@@ -278,7 +309,14 @@ reasoning about @('car') in general.</p>"
         (honsed-foo (da-honsed-constructor-name basename)))
     `(defun ,honsed-foo ,fields
        (declare (xargs :guard ,guard
-                       :guard-hints(("Goal" :in-theory (enable ,foo)))))
+                       ;; Same hints as for the ordinary constructor
+                       :guard-hints
+                       (("Goal"
+                         :in-theory (union-theories
+                                     '(,foo)
+                                     (theory 'minimal-theory)))
+                        (and stable-under-simplificationp
+                             '(:in-theory (enable ))))))
        (mbe :logic (,foo . ,fields)
             :exec ,(da-pack-fields t legiblep tag fields)))))
 
@@ -294,7 +332,25 @@ reasoning about @('car') in general.</p>"
          (fields-map (da-fields-map basename tag legiblep fields))
          (let-binds  (da-fields-map-let-bindings fields-map)))
   `(defund ,foo-p (,x)
-     (declare (xargs :guard t))
+     (declare (xargs :guard t
+                     :guard-hints
+                     (("Goal"
+                       :in-theory (union-theories
+                                   '((:executable-counterpart acl2::eqlablep)
+                                     acl2::consp-assoc-equal
+                                     acl2::assoc-eql-exec-is-assoc-equal)
+                                   (theory 'defaggregate-basic-theory)))
+                      (and stable-under-simplificationp
+                           ;; This looks like it does nothing, but the basic
+                           ;; effect is to undo the "goal" theory and go back
+                           ;; into the default theory.
+                           ;;
+                           ;; This is sometimes necessary because the later
+                           ;; requirements might have guards that depend on the
+                           ;; previous requirements.  The user needs to provide
+                           ;; a theory that is adequate to show this is the
+                           ;; case.
+                           '(:in-theory (enable ))))))
      (and ,@(if tag
                 `((consp ,x)
                   (eq (car ,x) ,tag))
@@ -315,7 +371,17 @@ reasoning about @('car') in general.</p>"
         (body     (cdr (assoc field map))))
   `(defund-inline ,foo->bar (,x)
      (declare (xargs :guard (,foo-p ,x)
-                     :guard-hints (("Goal" :in-theory (enable ,foo-p)))))
+                     :guard-hints (("Goal"
+                                    ;; expand hint sometimes needed due to mutual
+                                    ;; recursions
+                                    :expand (,foo-p ,x)
+                                    :in-theory
+                                    (union-theories
+                                     '(,foo-p
+                                       (:executable-counterpart acl2::eqlablep)
+                                       acl2::consp-assoc-equal
+                                       acl2::assoc-eql-exec-is-assoc-equal)
+                                     (theory 'defaggregate-basic-theory))))))
      ,body)))
 
 #||
@@ -347,7 +413,11 @@ reasoning about @('car') in general.</p>"
              basename)
      (equal (,foo->bar (,foo . ,all-fields))
             ,field)
-     :hints(("Goal" :in-theory (enable ,foo->bar ,foo))))))
+     :hints(("Goal"
+             :in-theory
+             (union-theories
+              '(,foo->bar ,foo)
+              (theory 'defaggregate-basic-theory)))))))
 
 (defun da-make-accessors-of-constructor-aux (basename fields all-fields)
   (if (consp fields)
@@ -474,46 +544,35 @@ reasoning about @('car') in general.</p>"
 
 (defun da-patbind-make-field-vars-alist (var fields)
   ;; Given var = 'foo and fields = '(a b c),
-  ;; Constructs '((a . foo.a) (b . foo.b) (c . foo.c))
+  ;; Constructs '(("FOO.A" . a) ("FOO.B" . b) ("FOO.C" . c))
   (if (atom fields)
       nil
-    (acons (car fields)
-           (intern-in-package-of-symbol
-            (concatenate 'string (symbol-name var) "." (symbol-name (car fields)))
-            var)
-          (da-patbind-make-field-vars-alist var (cdr fields)))))
+    (acons (concatenate 'string (symbol-name var) "." (symbol-name (car fields)))
+           (car fields)
+           (da-patbind-make-field-vars-alist var (cdr fields)))))
 
-(defun da-patbind-find-unused-vars (form vars)
-  ;; Return all vars not used in form.  We do this completely stupidly, not
-  ;; even avoiding quoted constants.  We can try to improve this if it's a
-  ;; problem, but at some level what we're trying to do is inherently broken
-  ;; anyway -- we just hope it's useful most of the time anyway.
+(defun da-patbind-find-used-vars (form varstrs acc)
+  ;; Varstrs is a list of strings such as "X.FOO" "X.BAR" etc.
+  ;; Acc accumulates (uniquely) all the symbols in FORM for which the
+  ;; symbol-name is in varstrs.
   (if (atom form)
-      (if (symbolp form)
-          (remove1 form vars)
-        vars)
-    (da-patbind-find-unused-vars (car form)
-                                 (da-patbind-find-unused-vars (cdr form) vars))))
+      (if (and (symbolp form)
+               (member-equal (symbol-name form) varstrs)
+               (not (member-eq form acc)))
+          (cons form acc)
+        acc)
+    (da-patbind-find-used-vars (car form) varstrs
+                               (da-patbind-find-used-vars (cdr form) varstrs acc))))
 
-;; (da-patbind-find-unused-vars '(foo (+ 1 a) c) '(a b c d)) --> '(b d)
-
-(defun da-patbind-remove-unused-vars (valist unused)
-  (cond ((atom valist)
-         nil)
-        ((member (cdar valist) unused)
-         (da-patbind-remove-unused-vars (cdr valist) unused))
-        (t
-         (cons (car valist)
-               (da-patbind-remove-unused-vars (cdr valist) unused)))))
-
-(defun da-patbind-alist-to-bindings (name valist target)
-  (if (atom valist)
+(defun da-patbind-alist-to-bindings (name vars valist target)
+  (if (atom vars)
       nil
-    (let* ((accessor (da-accessor-name name (caar valist)))
+    (let* ((fldname (cdr (assoc-equal (symbol-name (car vars)) valist)))
+           (accessor (da-accessor-name name fldname))
            (call     (list accessor target))     ;; (taco->shell foo)
-           (binding  (list (cdar valist) call))) ;; (x.foo (taco->shell foo))
+           (binding  (list (car vars) call))) ;; (x.foo (taco->shell foo))
       (cons binding
-            (da-patbind-alist-to-bindings name (cdr valist) target)))))
+            (da-patbind-alist-to-bindings name (cdr vars) valist target)))))
 
 
 (defun da-patbind-fn (name fields args forms rest-expr)
@@ -529,10 +588,9 @@ term.  The attempted binding of~|~% ~p1~%~%is not of this form."
 
        (var             (car args))
        (full-vars-alist (da-patbind-make-field-vars-alist var fields))
-       (field-vars      (strip-cdrs full-vars-alist))
-       (unused-vars     (da-patbind-find-unused-vars rest-expr field-vars))
-       (vars-alist      (da-patbind-remove-unused-vars full-vars-alist unused-vars))
-       ((unless vars-alist)
+       (field-vars      (strip-cars full-vars-alist))
+       (used-vars       (da-patbind-find-used-vars rest-expr field-vars nil))
+       ((unless used-vars)
         (progn$
          (cw "Note: not introducing any ~x0 field bindings for ~x1, since ~
               none of its fields appear to be used.~%" name var)
@@ -549,7 +607,7 @@ term.  The attempted binding of~|~% ~p1~%~%is not of this form."
        (binding  (if forms (car forms) var))
        (evaledp  (or (atom binding) (eq (car binding) 'quote)))
        (target   (if evaledp binding (acl2::pack binding)))
-       (bindings (da-patbind-alist-to-bindings name vars-alist target))
+       (bindings (da-patbind-alist-to-bindings name used-vars full-vars-alist target))
 
        ;;(- (cw "Binding is ~x0.~%" var))
        ;;(- (cw "Evaledp is ~x0.~%" var))
@@ -602,4 +660,4 @@ term.  The attempted binding of~|~% ~p1~%~%is not of this form."
      ((employee emp) emp))
   emp.name)
 
-||#
\ No newline at end of file
+||#
