@@ -23,7 +23,7 @@
 (in-package "CUTIL")
 (include-book "formals")
 (include-book "returnspecs")
-(include-book "xdoc-impl/fmt-to-str" :dir :system)
+(include-book "xdoc/fmt-to-str" :dir :system)
 (include-book "tools/mv-nth" :dir :system)
 (include-book "str/cat" :dir :system)
 (set-state-ok t)
@@ -42,7 +42,7 @@
 <ul>
 
 <li>Richer @('formals') lists that permit keyword/optional arguments, embedded
-guards and documentation, automatically infer @(see stobj) declarations,
+guards and documentation, automatically infer @(see acl2::stobj) declarations,
 etc.</li>
 
 <li>A more concise @(see xargs) syntax that also adds control over other
@@ -76,7 +76,7 @@ syntax and features to cover, and we address them in turn.</p>
 
 <p>The formal have many features; see @(see extended-formals).  Besides the
 ordinary extended-formals utilities, they can also include @(':type')
-declarations; see @(see type-spec).  For instance:</p>
+declarations; see @(see acl2::type-spec).  For instance:</p>
 
 @({
   (x oddp :type integer)
@@ -763,7 +763,11 @@ some kind of separator!</p>
                               :returns ',returnspecs
                               :formals ',formals)
 
-         (local (in-theory (enable ,fnname)))
+         (local
+          (make-event
+           (if (logic-mode-p ',fnname-fn (w state))
+               '(in-theory (enable ,fnname))
+             '(value-triple :invisible))))
 
          (make-event
           (let* ((world (w state))
