@@ -59,7 +59,7 @@
 
 (in-package "ACL2")
 
-(include-book "make-event/eval" :dir :system)
+(include-book "misc/eval" :dir :system)
 (include-book "cutil/defaggregate" :dir :system)
 (include-book "cutil/deflist" :dir :system)
 (include-book "misc/defun-plus" :dir :system)
@@ -256,6 +256,22 @@
 ; Server creates response
 ;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+; Reordering the rewrite-clause-type-alist: I added the uppercase text below to
+; make this work.  See the comment in rewrite-clause-type-alist.
+; JSM April 7, 2013.
+
+(DEFTHM LEMMA-ADDED-IN-TYPE-ALIST-REORDERING
+  (IMPLIES
+   (AND (NETWORK-STATE-P NETWORK-ST)
+        (CAR (RETRIEVE-NETWORK-MESSAGE :SERVER NETWORK-ST)))
+   (NETWORK-PACKET-P (CAR (RETRIEVE-NETWORK-MESSAGE :SERVER NETWORK-ST)))))
+
+; Note: The old proof of server-step1-output-lemma-2 (generated by the defun+
+; below) was quite complex: the string ``goal'' is mentioned 91 times, 3
+; inductions are performed, and there are 3 forcing rounds to deal with 7
+; forced subgoals.  The new proof, using the lemma above, mentions ``goal'' 12
+; times, does no inductions, and one subgoal is forced.
+
 (defun+ server-step1 (server-st network-st)
   (declare (xargs :guard (and (server-state-p server-st)
                               (network-state-p network-st))
