@@ -1522,7 +1522,7 @@ Example:
 
 (deflabel word/bit-macros
   :doc ":doc-section logops-definitions
-  Macros for manipulating integer words defined as contguous bits.
+  Macros for manipulating integer words defined as contiguous bits.
   ~/~/
   These macros were defined to support the functions produced by translating
   SPW .eqn files to ACL2 functions.~/")
@@ -1543,7 +1543,7 @@ Example:
 
   The above macro call will bind A, B, and C to the 0th, 1st, and 2nd bit
   of I, and then evaluate the logical expression under those bindings.
-  The list of bit names is always intrepreted from low to high order.~/~/"
+  The list of bit names is always interpreted from low to high order.~/~/"
 
   (declare (xargs :guard (and (symbol-listp bit-names)
 			      (no-duplicatesp bit-names))))
@@ -1559,7 +1559,7 @@ Example:
   (MAKE-WORD-FROM-BITS A B C)
 
   The above macro call will build an unsigned integer from the bits A
-  B, and C.  The list of bits is always intrepreted from low to high
+  B, and C.  The list of bits is always interpreted from low to high
   order. Note that the expression generated by this macro will coerce the
   values to bits before building the word.~/~/"
 
