@@ -1,9 +1,10 @@
-; Matt Kaufmann
-; August, 2007
+; Copyright (C) 2013, Regents of the University of Texas
+; Written by Matt Kaufmann, August, 2007
+; License: A 3-clause BSD license.  See the LICENSE file distributed with ACL2.
+
 ; Thanks to Robert Krug for requesting this tool.
 
 (in-package "ACL2")
-(include-book "doc-section")
 (include-book "bash")
 
 (program)
@@ -366,7 +367,7 @@
                             name call
                             &key hyp equiv hints debug (flatten 't))
 
-  ":Doc-Section misc
+  ":Doc-Section miscellaneous
 
   create a defthm equating a call with its simplification~/
 
