@@ -1,5 +1,14 @@
 (in-package "ACL2")
 
+#||
+
+; [Jared and Sol]: fool make_cert_help.pl into allowing more memory for this
+; book. We would just include centaur/misc/memory-mgmt, but that has a ttag.
+
+(set-max-mem (* 6 (expt 2 30)))
+
+||#
+
 ; All theorems in this file are local except for those that are to be exported,
 ; which are labeled with the comment "; Exported theorem:".  We use def-gl-thm
 ; (actually a local version) so dispatch lemmas that were seen to be needed
@@ -705,6 +714,10 @@
   :g-bindings
   `((addr (:g-number ,(gl-int 0 1 33)))))
 
+; Reordering the rewrite-clause-type-alist: I added the uppercase text below to
+; make this work.  See the comment in rewrite-clause-type-alist.
+; JSM April 7, 2013.
+
 (defthm rm32-rm08
   (implies (and (force (x86-32p x86-32))
                 (force (n32p addr))
@@ -717,7 +730,10 @@
                           (ash (rm08 (+ 1 addr) x86-32)
                                8)
                           (rm08 addr x86-32))))
-  :hints (("Goal" :in-theory (enable rm32 rm08))))
+  :hints (("Goal"
+           :in-theory
+           (e/d (rm32 rm08)
+                ((:EXECUTABLE-COUNTERPART BINARY-LOGAND))))))
 
 ; In order to prove rules about the case of overlapping addresses for 32-bit
 ; read/write, our strategy is to reduce reads to 8-bit reads and to reason
