@@ -10,6 +10,7 @@
 ||#
 
 (include-book "x86-state")
+(include-book "std/lists/list-defuns" :dir :system)
 
 (local
  (defmacro enable-stobj-fns-fn (x86-32-model)
@@ -762,6 +763,8 @@
            (good-mem-arrayp-1-logic index2 len mem-array)))
 
 
+;; (local (in-theory (disable nth-when-zp)))
+
 (encapsulate
  ()
 
@@ -887,31 +890,40 @@
 
 ; Start proof of x86-32p-!memi-new-page-resize
 
+(defthm mem-arrayp-repeat
+  (implies (unsigned-byte-p 8 x)
+           (mem-arrayp (repeat x n)))
+  :hints(("Goal" :in-theory (enable repeat))))
+
 (defthm mem-arrayp-resize-list
   (implies (and (mem-arrayp lst)
                 (unsigned-byte-p 8 default-value))
-           (mem-arrayp (resize-list lst new-len default-value))))
+           (mem-arrayp (resize-list lst new-len default-value)))
+  :hints(("Goal" :in-theory (enable resize-list))))
 
-(defun nth-resize-list-induction (i lst n default-value)
-  (declare (ignorable i lst n default-value))
-  (if (posp n)
-      (nth-resize-list-induction (1- i)
-                                 (if (atom lst) lst (cdr lst))
-                                 (1- n)
-                                 default-value)
-    nil))
 
-(defthm nth-resize-list
-  (implies (and (natp i)
-                (natp n)
-                (<= (len lst) n)
-                (< i n))
-           (equal (nth i (resize-list lst n default))
-                  (if (< i (len lst))
-                      (nth i lst)
-                    default)))
-  :hints (("Goal" :in-theory (enable resize-list nth)
-           :induct (nth-resize-list-induction i lst n default-value))))
+(local
+ (progn
+   (defun nth-resize-list-induction (i lst n default-value)
+     (declare (ignorable i lst n default-value))
+     (if (posp n)
+         (nth-resize-list-induction (1- i)
+                                    (if (atom lst) lst (cdr lst))
+                                    (1- n)
+                                    default-value)
+       nil))
+
+   (defthm nth-resize-list
+     (implies (and (natp i)
+                   (natp n)
+                   (<= (len lst) n)
+                   (< i n))
+              (equal (nth i (resize-list lst n default))
+                     (if (< i (len lst))
+                         (nth i lst)
+                       default)))
+     :hints (("Goal" :in-theory (enable resize-list nth)
+              :induct (nth-resize-list-induction i lst n default-value))))))
 
 (defthm good-mem-arrayp-1-logic-resize-list
   (implies (and (natp next-addr)
@@ -1662,19 +1674,30 @@
                                         (NTH *MEM-TABLEI* X86-32)))
                   0)))
 
-(defthm nth-resize-list++
-  (implies (and (natp i)
-                (natp n)
-                (<= (len lst) n))
-           (equal (nth i (resize-list lst n default))
-                  (cond ((< i (len lst))
-                         (nth i lst))
-                        ((< i n)
-                         default)
-                        (t
-                         nil))))
-  :hints (("Goal" :in-theory (enable resize-list nth)
-           :induct (nth-resize-list-induction i lst n default-value))))
+;; (defthm nth-resize-list++
+;;   (implies (and (natp i)
+;;                 (natp n)
+;;                 (<= (len lst) n))
+;;            (equal (nth i (resize-list lst n default))
+;;                   (cond ((< i (len lst))
+;;                          (nth i lst))
+;;                         ((< i n)
+;;                          default)
+;;                         (t
+;;                          nil))))
+;;   :hints (("Goal" :in-theory (enable resize-list nth)
+;;            :induct (nth-resize-list-induction i lst n default-value))))
+
+; At least some of the following enables and disables are required on 7/28/2013
+; for read-write, perhaps due to a change in or under centaur/books/gl/gl.lisp
+; -- actually the culprit is at least in part probably the additions of some
+; rules to centaur/misc/arith-equivs.lisp.
+;; (local (in-theory (e/d (nfix natp)
+;;                        (natp-when-gte-0
+;;                         natp-when-integerp
+;;                         nfix-when-natp
+;;                         nfix-when-not-natp
+;;                         nth-when-too-large))))
 
 (encapsulate
  ()
@@ -1967,4 +1990,4 @@
 
 (in-theory (disable wm32))
 
-; ======================================================================
\ No newline at end of file
+; ======================================================================
