@@ -1,5 +1,11 @@
 (in-package "ACL2")
 
+; Added by Matt K.:
+; Require ACL2(h) or ACL2(hp) because of the use of GL in this directory.
+; cert_param: (hons-only)
+(local (include-book "misc/check-state" :dir :system))
+(local (check-hons-enabled (:book "des.lisp")))
+
 (include-book "centaur/gl/gl" :dir :system)
 (include-book "security/util/byte-operations" :dir :system)
 
