@@ -1,4 +1,5 @@
 ; Standard IO Library
+; base.lisp
 ; Copyright (C) 2008-2013 Centaur Technology
 ;
 ; Contact:
@@ -18,11 +19,17 @@
 ;
 ; Original author: Sol Swords <sswords@centtech.com>
 ; Documented by Jared Davis <jared@centtech.com>
-; Moved by David Rager <ragerdl@gmail.com> on 1/10/13.
+; Moved into std by David Rager <ragerdl@gmail.com>
 
 (in-package "ACL2")
 (include-book "xdoc/top" :dir :system)
 (include-book "system/f-put-global" :dir :system)
+(include-book "std/lists/list-defuns" :dir :system)
+(include-book "file-measure")
+(local (include-book "std/lists/append" :dir :system))
+(local (include-book "std/lists/revappend" :dir :system))
+(local (include-book "std/lists/rev" :dir :system))
+(local (include-book "str/coerce" :dir :system))
 
 (defsection bytep
 
@@ -65,21 +72,23 @@
 
 (local (table evisc-table (list 'quote *INITIAL-GLOBAL-TABLE*) "*INITIAL-GLOBAL-TABLE*"))
 
-(defthm assoc-equal-of-add-pair
-    (equal (assoc-equal k1 (add-pair k2 v a))
-           (if (equal k1 k2)
-               (cons k2 v)
-             (assoc-equal k1 a))))
+;; [Jared] Removed because this is now built into ACL2
+;; (defthm assoc-equal-of-add-pair
+;;     (equal (assoc-equal k1 (add-pair k2 v a))
+;;            (if (equal k1 k2)
+;;                (cons k2 v)
+;;              (assoc-equal k1 a))))
 
 (defthm open-channel-listp-of-add-pair
   (implies (and (open-channel1 v)
                 (open-channel-listp x))
            (open-channel-listp (add-pair k v x))))
 
-(defthm ordered-symbol-alistp-of-add-pair
-  (implies (and (symbolp k)
-                (ordered-symbol-alistp x))
-           (ordered-symbol-alistp (add-pair k v x))))
+;; [Jared] Removed because this is now built into ACL2
+;; (defthm ordered-symbol-alistp-of-add-pair
+;;   (implies (and (symbolp k)
+;;                 (ordered-symbol-alistp x))
+;;            (ordered-symbol-alistp (add-pair k v x))))
 
 (defthm open-channels-p-of-add-pair
   (implies (and (symbolp k)
@@ -103,32 +112,13 @@
   :hints(("Goal" :in-theory (enable readable-files-p))))
 
 (defxdoc *file-types*
-  :parents (io)
+  :parents (std/io)
   :short "Different ways to open files for reading and writing."
 
   :long "<p>See @(see io) for general discussion of file types.</p>
 
 @(def *file-types*)")
 
-(defxdoc logical-story-of-io
-  :parents (io)
-  :short "How file reading operations are modeled in the ACL2 logic."
-
-  :long "<p>ACL2's file input operations are available in @(':logic') mode (see
-@(see defun-mode)).  This is somewhat tricky to justify in the logic, since,
-e.g., the contents of a file is external to ACL2, can be changed over time, and
-so on.</p>
-
-<p>Practically speaking, most users don't need to pay any attention to the
-details of this story.  Instead, see the book @('io/base'), which develops
-the basic theorems that are necessary to reason about the io primitives.</p>
-
-<p>If, for some reason, you do want to understand the logical story, you might
-start with this paper:</p>
-
-<p>Jared Davis.  <a
-href='http://www.ccs.neu.edu/home/pete/acl206/papers/davisb.pdf'>Reasoning
-about ACL2 File Input</a>.  ACL2 Workshop 2006.</p>")
 
 
 
@@ -139,9 +129,9 @@ about ACL2 File Input</a>.  ACL2 Workshop 2006.</p>")
 ;
 ; -----------------------------------------------------------------------------
 
-(defsection open-input-channel
-  :parents (io)
-  :short "Open a file for reading."
+(defsection std/io/open-input-channel
+  :parents (std/io open-input-channel)
+  :short "Open an file for reading."
 
   :long "<p><b>Signature:</b> @(call open-input-channel) returns @('(mv channel
 state)')</p>
@@ -208,157 +198,8 @@ explanation of why opening the file failed.</p>"
               (mv-nth 1 (open-input-channel fname type state))))
     :hints(("Goal" :in-theory (enable state-p1))))
 
-  ;; We now show that the channel returned by OPEN-INPUT-CHANNEL is never
-  ;;
-  ;;   - ACL2-INPUT-CHANNEL::STANDARD-CHARACTER-INPUT-0, or
-  ;;   - ACL2-INPUT-CHANNEL::STANDARD-OBJECT-INPUT-0.
-  ;;
-  ;; These facts are needed to verify the guards on CLOSE-INPUT-CHANNEL after
-  ;; you open an input channel.
 
-  (local (include-book "std/misc/explode-nonnegative-integer" :dir :system))
-  (local (include-book "std/misc/intern-in-package-of-symbol" :dir :system))
-  (local (include-book "std/lists/coerce" :dir :system))
-
-  (local (defthm lemma-0
-           (implies (and (base10-digit-char-listp y1)
-                         (base10-digit-char-listp y2)
-                         (not (base10-digit-charp x)))
-                    (equal (equal (cons x y1)
-                                  (cons x y2))
-                           (equal y1 y2)))))
-
-  (local (defthm lemma-1
-           (implies (and (base10-digit-char-listp y1)
-                         (base10-digit-char-listp y2)
-                         (not (base10-digit-charp x)))
-                    (not (equal (cons a (cons x y1))
-                                (cons x y2))))))
-
-  (local (defthm lemma-2
-           (implies (and (base10-digit-char-listp y1)
-                         (base10-digit-char-listp y2)
-                         (not (base10-digit-charp x))
-                         (equal (append a (cons x y1))
-                                (cons x y2)))
-                    (not (consp a)))
-           :hints(("Goal" :induct (append a (cons x y1))
-                   :do-not '(generalize)))))
-
-  (local (defthm lemma-3
-           (implies (and (base10-digit-char-listp y1)
-                         (base10-digit-char-listp y2)
-                         (not (base10-digit-charp x)))
-                    (equal (equal (append a (cons x y1))
-                                  (cons x y2))
-                           (and (not (consp a))
-                                (equal y1 y2))))
-           :hints(("Goal"
-                   :in-theory (disable lemma-2)
-                   :use (:instance lemma-2 (a a) (x x) (y1 y1) (y2 y2))))))
-
-  (local (defun cdr-cdr-induction (a1 a2)
-           (if (and (consp a1)
-                    (consp a2))
-               (cdr-cdr-induction (cdr a1) (cdr a2))
-             (list a1 a2))))
-
-  (local (defthm lemma-4
-           (implies (and (base10-digit-char-listp y1)
-                         (base10-digit-char-listp y2)
-                         (not (base10-digit-charp x))
-                         (true-listp a1)
-                         (true-listp a2))
-                    (equal (equal (append a1 (cons x y1))
-                                  (append a2 (cons x y2)))
-                           (and (equal y1 y2)
-                                (equal a1 a2))))
-           :hints(("Goal" :induct (cdr-cdr-induction a1 a2)))))
-
-  (local (defthm lemma-5
-           (implies (and (base10-digit-char-listp y1)
-                         (base10-digit-char-listp y2)
-                         (true-listp x1)
-                         (true-listp x2))
-                    (equal (equal (append x1 (cons #\- y1))
-                                  (append x2 (cons #\- y2)))
-                           (and (equal x1 x2)
-                                (equal y1 y2))))))
-
-  (local (defthm lemma-6
-           (implies (and (true-listp x)
-                         (natp clock))
-                    (equal (equal (append x (cons #\- (explode-atom clock 10)))
-                                  (append x (cons #\- (explode-atom 0 10))))
-                           (equal clock 0)))))
-
-  (local (defthm main-lemma
-           (implies (and (natp clock)
-                         (character-listp x))
-                    (equal (equal (make-input-channel filename clock)
-                                  (intern-in-package-of-symbol
-                                   (coerce (append x (cons #\- (explode-atom 0 10)))
-                                           'string)
-                                   'ACL2-INPUT-CHANNEL::A-RANDOM-SYMBOL-FOR-INTERN))
-                           (and (equal (coerce filename 'list) x)
-                                (equal clock 0))))
-           :hints(("Goal" :in-theory (disable (explode-atom))))))
-
-  (local (defthm coerce-reduction
-           (implies (and (syntaxp (quotep list))
-                         (character-listp list)
-                         (stringp x))
-                    (equal (equal (coerce x 'list) list)
-                           (equal x (coerce list 'string))))))
-
-  (local (defthm make-input-channel-standard-character-input-0
-           (implies (and (stringp filename)
-                         (natp clock))
-                    (equal (equal (make-input-channel filename clock)
-                                  'ACL2-INPUT-CHANNEL::STANDARD-CHARACTER-INPUT-0)
-                           (and (equal filename "STANDARD-CHARACTER-INPUT")
-                                (equal clock 0))))
-           :hints(("Goal"
-                   :in-theory (disable main-lemma)
-                   :use ((:instance main-lemma
-                                    (clock clock)
-                                    (filename filename)
-                                    (x (coerce "STANDARD-CHARACTER-INPUT" 'list))))))))
-
-  (local (defthm make-input-channel-standard-object-input-0
-           (implies (and (stringp filename)
-                         (natp clock))
-                    (equal (equal (make-input-channel filename clock)
-                                  'ACL2-INPUT-CHANNEL::STANDARD-OBJECT-INPUT-0)
-                           (and (equal filename "STANDARD-OBJECT-INPUT")
-                                (equal clock 0))))
-           :hints(("Goal"
-                   :in-theory (disable main-lemma)
-                   :use ((:instance main-lemma
-                                    (clock clock)
-                                    (filename filename)
-                                    (x (coerce "STANDARD-OBJECT-INPUT" 'list))))))))
-
-
-  (local (defthm open-input-channel-channel-elim
-           (implies (and (mv-nth 0 (open-input-channel filename type state))
-                         (state-p1 state))
-                    (equal (mv-nth 0 (open-input-channel filename type state))
-                           (make-input-channel filename (1+ (file-clock state)))))))
-
-  (defthm open-input-channel-not-standard-object-input
-    (implies (and (stringp filename)
-                  (state-p1 state))
-             (not (equal (mv-nth 0 (open-input-channel filename type state))
-                         'ACL2-INPUT-CHANNEL::STANDARD-OBJECT-INPUT-0)))
-    :hints(("Goal" :in-theory (disable make-input-channel))))
-
-  (defthm open-input-channel-not-standard-character-input
-    (implies (and (stringp filename)
-                  (state-p1 state))
-             (not (equal (mv-nth 0 (open-input-channel filename type state))
-                         'ACL2-INPUT-CHANNEL::STANDARD-CHARACTER-INPUT-0)))
-    :hints(("Goal" :in-theory (disable make-input-channel)))))
+  )
 
 
 ;; helper theorems for reading
@@ -385,8 +226,8 @@ explanation of why opening the file failed.</p>"
 
 
 
-(defsection close-input-channel
-  :parents (io)
+(defsection std/io/close-input-channel
+  :parents (std/io close-input-channel)
   :short "Close an input channel."
 
   :long "<p><b>Signature</b>: @(call close-input-channel) returns
@@ -441,8 +282,8 @@ them to avoid resource leaks.</p>"
                                        open-input-channel-p1
                                        open-input-channel)))))
 
-(defsection read-char$
-  :parents (io)
+(defsection std/io/read-char$
+  :parents (std/io read-char$)
   :short "Read one character from an open @(':character') stream."
 
   :long "<p>NOTE: We generally prefer to avoid @(see read-char$).  It is easy
@@ -566,8 +407,8 @@ input streams.  So, that might be worth considering.</p>"
                                   (or (characterp (mv-nth 0 (read-char$ channel state)))
                                       (null (mv-nth 0 (read-char$ channel state)))))))))
 
-(defsection peek-char$
-  :parents (io)
+(defsection std/io/peek-char$
+  :parents (std/io peek-char$)
   :short "Inspect the next character that will be read from an open
 @(':character') input stream."
 
@@ -600,8 +441,8 @@ reason about @('read-char$') instead.</p>"
   (in-theory (disable peek-char$)))
 
 
-(defsection read-byte$
-  :parents (io)
+(defsection std/io/read-byte$
+  :parents (std/io read-byte$)
   :short "Read one byte from an open @(':byte') stream."
 
   :long "<p><b>Signature:</b> @(call read-byte$) returns @('(mv byte/nil
@@ -716,8 +557,8 @@ the end of the file, @('nil') is returned.</p>"
     :hints(("Goal" :in-theory (disable open-input-channel-p1)))))
 
 
-(defsection read-object
-  :parents (io)
+(defsection std/io/read-object
+  :parents (std/io read-object)
   :short "Read one object from an open @(':object') stream."
 
   :long "<p><b>Signature:</b> @(call read-object) returns @('(mv eofp obj
@@ -825,7 +666,7 @@ foo
           :hints(("Goal" :in-theory (enable nth update-nth)))))
 
  (defsection char
-   :extension read-char$
+   :extension std/io/read-char$
    (defthm state-preserved-by-read-char$-when-eof
      (implies (and (not (mv-nth 0 (read-char$ channel state)))
                    (state-p1 state)
@@ -835,7 +676,7 @@ foo
      :hints(("Goal" :in-theory (e/d (read-char$ state-p1))))))
 
  (defsection byte
-   :extension read-byte$
+   :extension std/io/read-byte$
    (defthm state-preserved-by-read-byte$-when-eof
      (implies (and (not (mv-nth 0 (read-byte$ channel state)))
                    (state-p1 state)
@@ -845,7 +686,7 @@ foo
      :hints(("Goal" :in-theory (e/d (read-byte$ state-p1))))))
 
  (defsection object
-   :extension read-object$
+   :extension std/io/read-object$
    (defthm state-preserved-by-read-object-when-eof
      (implies (and (mv-nth 0 (read-object channel state))
                    (state-p1 state)
@@ -862,8 +703,8 @@ foo
 ;
 ; -----------------------------------------------------------------------------
 
-(defsection open-output-channel
-  :parents (io)
+(defsection std/io/open-output-channel
+  :parents (std/io open-output-channel)
   :short "Open a file for writing."
 
   :long "<p><b>Signature:</b> @(call open-output-channel) returns @('(mv
@@ -955,8 +796,8 @@ created.</li>
 
 
 
-(defsection close-output-channel
-  :parents (io)
+(defsection std/io/close-output-channel
+  :parents (std/io close-output-channel)
   :short "Close an output channel."
 
   :long "<p><b>Signature</b>: @(call close-output-channel) returns
@@ -1027,35 +868,31 @@ output channels when you are done with them to avoid resource leaks.</p>"
                 (equal (cadar x) :object))
            (open-channel1 (list* (car x) y (cdr x)))))
 
+(defthm typed-io-listp-of-append
+  (equal (typed-io-listp (append x y) type)
+         (and (typed-io-listp (list-fix x) type)
+              (typed-io-listp y type))))
+
+(defthm typed-io-listp-of-rev
+  (equal (typed-io-listp (rev x) type)
+         (typed-io-listp (list-fix x) type))
+  :hints(("Goal" :in-theory (enable rev))))
+
 (defthm open-channel1-of-revappend-charlist
   (implies (and (open-channel1 x)
                 (equal (cadar x) :character)
                 (character-listp y))
-           (open-channel1 (cons (car x) (revappend y (cdr x)))))
-  :otf-flg t)
-
-;; matches unicode/explode-nonnegative-integer.lisp
-(defthm character-listp-of-explode-nonnegative-integer
-  (equal (character-listp (explode-nonnegative-integer n base acc))
-         (character-listp acc))
-  :hints(("Goal" :in-theory (disable floor mod digit-to-char))))
-
-;; does NOT match unicode/explode-atom.lisp (so we renamed this theorem).
-;; unicode version includes print-base-p hyp.
-(defthm character-listp-explode-atom
-  (character-listp (explode-atom x base)))
-
-; Added by Matt K. for princ$ change 12/7/2012.
-(defthm character-listp-explode-atom+
-  (character-listp (explode-atom+ x base radix)))
+           (open-channel1 (cons (car x) (append (rev y) (cdr x))))))
 
 
+(defsection std/io/princ$
+  :parents (std/io princ$)
+  :short "Print an atom to a @(':character') output stream."
+  :long "<p>ACL2 has nice documentation for @(see princ$).  The @(see std/io)
+library adds the following lemmas:</p>"
 
-;; I-AM-HERE
-(defsection princ$
-  :parents (io)
-  ;; BOZO ACL2's documentation for princ$ is nice, it'd be good to have a better
-  ;; integration between it and XDOC.
+  (local (include-book "str/explode-nonnegative-integer" :dir :system))
+  (local (include-book "str/explode-atom" :dir :system))
 
   (defthm state-p1-of-princ$
     (implies (and (state-p1 state)
@@ -1067,13 +904,33 @@ output channels when you are done with them to avoid resource leaks.</p>"
   (defthm open-output-channel-p1-of-princ$
     (implies (and (state-p1 state)
                   (open-output-channel-p1 channel :character state))
-             (open-output-channel-p1 channel :character (princ$ x channel state)))))
+             (open-output-channel-p1 channel :character
+                                     (princ$ x channel state)))))
 
 
-(defsection write-byte$
-  :parents (io)
+(defsection std/io/write-byte$
+  :parents (std/io write-byte$)
+  :short "Write one byte to an open @(':byte') output stream."
+  :long "<p><b>Signature:</b> @(call write-byte$) returns @('state').</p>
+
+<ul>
+
+<li>@('byte') must be a natural number with @('byte < 256').</li>
+
+<li>@('channel') is a symbol that must refer to an open @(':byte') input
+channel; see @(see open-input-channel).</li>
+
+<li>@('state') is the ACL2 @(see state).</li>
+
+</ul>
+
+<p>This is a @(see logic)-mode function, but its logical definition is tricky;
+see @(see logical-story-of-io).  The main logical consequence is the updating
+of state.</p>
+
+<p>Under the hood, we write a byte to the Lisp output stream that is associated
+with @('channel').</p>"
 
-  ;; Write-byte$
   (defthm state-p1-of-write-byte$
     (implies (and (state-p1 state)
                   (symbolp channel)
@@ -1086,13 +943,15 @@ output channels when you are done with them to avoid resource leaks.</p>"
   (defthm open-output-channel-p1-of-write-byte$
     (implies (and (state-p1 state)
                   (open-output-channel-p1 channel :byte state))
-             (open-output-channel-p1 channel :byte (write-byte$ byte channel state)))))
+             (open-output-channel-p1 channel :byte
+                                     (write-byte$ byte channel state)))))
 
 
-(defsection print-object$
-  :parents (io)
+(defsection std/io/print-object$
+  :parents (std/io print-object$)
+  :short "Print a Lisp object to an @(':object') output stream."
+  :long "<p>BOZO document me.</p>"
 
-  ;; print-object$
   (defthm state-p1-of-print-object$
     (implies (and (state-p1 state)
                   (symbolp channel)
@@ -1103,16 +962,21 @@ output channels when you are done with them to avoid resource leaks.</p>"
   (defthm open-output-channel-p1-of-print-object$
     (implies (and (state-p1 state)
                   (open-output-channel-p1 channel :object state))
-             (open-output-channel-p1 channel :object (print-object$ x channel state)))))
+             (open-output-channel-p1 channel
+                                     :object (print-object$ x channel state)))))
 
 
 
-;; Close-output-channel's guard requires that the channel is not the symbol:
-;; ACL2-OUTPUT-CHANNEL::STANDARD-CHARACTER-OUTPUT-0.
-;; So we need to prove that this symbol is never returned from
-;; open-output-channel.  This is true because symbol created ends in "-" and
-;; then the new file clock, which is at least 1.  Blech!
-(encapsulate nil
+
+(defsection open-channels-distinct-from-standard-io
+  :parents (open-input-channel open-output-channel)
+  :short "Technical lemmas to show that opening input/output channels does not
+produce a symbol like @('ACL2-OUTPUT-CHANNEL::STANDARD-CHARACTER-OUTPUT-0'),
+for the guards of @(see close-input-channel) and @(see close-output-channel)."
+
+;; This is true because symbol created ends in "-" and then the new file clock,
+;; which is at least 1.  Blech!
+
   (local (in-theory (disable floor mod print-base-p)))
   (local (include-book "ihs/quotient-remainder-lemmas" :dir :system))
 
@@ -1163,7 +1027,7 @@ output channels when you are done with them to avoid resource leaks.</p>"
            :hints(("Goal" ; :in-theory (enable mod)
                    :expand ((:free (base) (explode-nonnegative-integer x base ans)))
                    :use ((:instance explode-nonnegative-integer-length-incr-strict
-                          (n (* (/ base) x)) (ans (cons #\0 ans))))
+                                    (n (* (/ base) x)) (ans (cons #\0 ans))))
                    :do-not-induct t))
            :otf-flg t))
 
@@ -1173,7 +1037,7 @@ output channels when you are done with them to avoid resource leaks.</p>"
                     (charlist-suffixp (cdr x) suff)))))
 
   (local (defun symb-ends-in-dash-zero (x)
-           (charlist-suffixp (coerce (symbol-name x) 'list)
+           (charlist-suffixp (explode (symbol-name x))
                              '(#\- #\0))))
 
   (local (defthm suffixp-dash-zero-implies-member-dash
@@ -1250,92 +1114,6 @@ output channels when you are done with them to avoid resource leaks.</p>"
     :hints(("Goal" :in-theory (disable make-input-channel)))))
 
 
-(defsection file-measure
-  :parents (io)
-  :short "A measure for admitting functions that read from files."
-
-  :long "<p><b>Signature:</b> @(call file-measure) returns a natural
-number.</p>
-
-<ul>
-
-<li>@('channel') may be any symbol but is typically an open input channel.</li>
-
-<li>@('state-state') is typically the ACL2 @(see state).</li>
-
-</ul>
-
-<p>This is a @(see logic)-mode function, but its logical definition is tricky;
-see @(see logical-story-of-io).  The basic gist is that it returns how many
-objects are left in the channel and hence how many functions can still be
-read.</p>
-
-<p>This function is only meant to be used to admit functions, and cannot be
-executed on the real ACL2 @(see state).</p>
-
-<p>History.  Jared wrote an initial version of this function for the Unicode
-books.  Sol then extended it with a hack that allows us to prove it decreasing
-without assuming state-p1 in the object case.  (Really it's just a workaround
-for the fact that read-object checks @('(cdr entry)') as a substitute for
-@('(consp (cdr entry))') to find whether there are objects remaining.</p>"
-
-  (defun file-measure (channel state-state)
-    (declare (xargs :guard (and (symbolp channel)
-                                (state-p1 state-state))))
-    (+ (len (cddr (assoc-equal channel (open-input-channels state-state))))
-       (if (consp (cddr (assoc-equal channel (open-input-channels state-state))))
-           (if (cdr (last (cddr (assoc-equal channel (open-input-channels
-                                                      state-state))))) 1 0)
-         (if (cddr (assoc-equal channel (open-input-channels state-state))) 1 0))))
-
-  (defthm file-measure-of-read-byte$-weak
-    (<= (file-measure channel (mv-nth 1 (read-byte$ channel state)))
-        (file-measure channel state))
-    :rule-classes (:rewrite :linear))
-
-  (defthm file-measure-of-read-byte$-strong
-    (implies (mv-nth 0 (read-byte$ channel state))
-             (< (file-measure channel (mv-nth 1 (read-byte$ channel state)))
-                (file-measure channel state)))
-    :rule-classes (:rewrite :linear))
-
-  (defthm file-measure-of-read-byte$-rw
-    (implies (mv-nth 0 (read-byte$ channel state))
-             (equal (file-measure channel (mv-nth 1 (read-byte$ channel state)))
-                    (+ -1 (file-measure channel state)))))
-
-  (defthm file-measure-of-read-char$-weak
-    (<= (file-measure channel (mv-nth 1 (read-char$ channel state)))
-        (file-measure channel state))
-    :rule-classes (:rewrite :linear))
-
-  (defthm file-measure-of-read-char$-strong
-    (implies (mv-nth 0 (read-char$ channel state))
-             (< (file-measure channel (mv-nth 1 (read-char$ channel state)))
-                (file-measure channel state)))
-    :rule-classes (:rewrite :linear))
-
-  (defthm file-measure-of-read-char$-rw
-    (implies (mv-nth 0 (read-char$ channel state))
-             (equal (file-measure channel (mv-nth 1 (read-char$ channel state)))
-                    (1- (file-measure channel state)))))
-
-  (defthm file-measure-of-read-object-weak
-    (<= (file-measure channel (mv-nth 2 (read-object channel state)))
-        (file-measure channel state))
-    :rule-classes (:rewrite :linear))
-
-  (defthm file-measure-of-read-object-strong
-    (implies (not (mv-nth 0 (read-object channel state)))
-             (< (file-measure channel (mv-nth 2 (read-object channel state)))
-                (file-measure channel state)))
-    :rule-classes (:rewrite :linear))
-
-  (defthm file-measure-of-read-object-rw
-    (implies (not (mv-nth 0 (read-object channel state)))
-             (equal (file-measure channel (mv-nth 2 (read-object channel state)))
-                    (1- (file-measure channel state))))))
-
 
 (in-theory (disable state-p1
                     open-input-channel-p1
