@@ -23,6 +23,7 @@
 (include-book "equiv")
 (include-book "mfc-utils")
 (include-book "rev")
+(include-book "rcons")
 (include-book "flatten")
 
 (local (defthm equal-of-booleans-to-iff
@@ -132,7 +133,7 @@ library.")
                (subsetp x y)
              nil)))
 
-  (defthmd subsetp-member
+  (defthm subsetp-member
     (implies (and (member a x)
                   (subsetp x y))
              (member a y))
@@ -229,7 +230,7 @@ about set intersection.</p>"
              (subsetp x z))
     :hints(("Goal" :in-theory (enable subsetp-member))))
 
-  (defthmd subsetp-trans2
+  (defthm subsetp-trans2
     (implies (and (subsetp y z)
                   (subsetp x y))
              (subsetp x z))
@@ -571,7 +572,7 @@ about set equivalence.</p>"
   (implies (subsetp x y)
            (subsetp (remove a x) y)))
 
-(defcong set-equiv set-equiv (remove k a) 2
+(defcong set-equiv set-equiv (remove a x) 2
   :hints(("Goal" :in-theory (enable set-equiv))))
 
 (defthm rev-under-set-equiv
@@ -607,3 +608,9 @@ about set equivalence.</p>"
   (defcong set-equiv set-equiv (flatten x) 1
     :hints(("Goal" :in-theory (enable set-equiv)))))
 
+
+(defthm rcons-under-set-equiv
+  (set-equiv (rcons a x)
+             (cons a x))
+  :hints(("Goal" :in-theory (enable rcons))))
+
