@@ -1,5 +1,5 @@
 ; ACL2 String Library
-; Copyright (C) 2009-2010 Centaur Technology
+; Copyright (C) 2009-2013 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
@@ -36,7 +36,7 @@
 
 
 (defsection up-alpha-p
-  :parents (case-conversion acl2::upper-case-p)
+  :parents (cases acl2::upper-case-p)
   :short "Determine if a character is an upper-case letter (A-Z)."
 
   :long "<p>@(call up-alpha-p) determines if @('x') is an upper-case alphabetic
@@ -111,12 +111,15 @@ contrast, @('up-alpha-p') works on arbitrary characters.</p>"
 
   (defthm upper-case-p-is-up-alpha-p
     (equal (acl2::upper-case-p x)
-           (up-alpha-p (double-rewrite x)))))
+           (up-alpha-p (double-rewrite x))))
+
+  ;; No longer necessary since we'll rewrite upper-case-p away
+  (in-theory (disable acl2::upper-case-p-char-upcase)))
 
 
 
 (defsection down-alpha-p
-  :parents (case-conversion acl2::lower-case-p)
+  :parents (cases acl2::lower-case-p)
   :short "Determine if a character is a lower-case letter (a-z)."
 
   :long "<p>@(call down-alpha-p) determines if @('x') is an lower-case
@@ -191,12 +194,21 @@ contrast, @('down-alpha-p') works on arbitrary characters.</p>"
 
   (defthm lower-case-p-is-down-alpha-p
     (equal (acl2::lower-case-p x)
-           (down-alpha-p (double-rewrite x)))))
+           (down-alpha-p (double-rewrite x))))
+
+  ;; No longer necessary since we'll rewrite lower-case-p away
+  (in-theory (disable acl2::lower-case-p-char-downcase))
+
+  (defthm down-alpha-p-when-up-alpha-p
+    (implies (up-alpha-p x)
+             (not (down-alpha-p x)))
+    :hints(("Goal" :in-theory (enable up-alpha-p
+                                      down-alpha-p)))))
 
 
 
 (defsection upcase-char
-  :parents (case-conversion acl2::char-upcase)
+  :parents (cases acl2::char-upcase)
   :short "Convert a character to upper-case."
 
   :long "<p>@(call upcase-char) converts lower-case characters into their
@@ -224,6 +236,10 @@ contrast, @('upcase-char') works on arbitrary characters.</p>"
                     (char-fix x)))
     :hints(("Goal" :in-theory (enable down-alpha-p))))
 
+  (defthm upcase-char-of-upcase-char
+    (equal (upcase-char (upcase-char x))
+           (upcase-char x)))
+
   ;; Rewrite ACL2's char-upcase to upcase-char.  It seems simplest to just do
   ;; the proof by exhaustive testing.
 
@@ -293,8 +309,9 @@ contrast, @('upcase-char') works on arbitrary characters.</p>"
 
 
 
+
 (defsection downcase-char
-  :parents (case-conversion acl2::char-downcase)
+  :parents (cases acl2::char-downcase)
   :short "Convert a character to lower-case."
 
   :long "<p>@(call downcase-char) converts upper-case characters into their
@@ -323,6 +340,22 @@ characters.</p>"
                     (char-fix x)))
     :hints(("Goal" :in-theory (enable up-alpha-p))))
 
+  (defthm downcase-char-of-downcase-char
+    (equal (downcase-char (downcase-char x))
+           (downcase-char x)))
+
+  (defthm downcase-char-of-upcase-char
+    (equal (downcase-char (upcase-char x))
+           (downcase-char x))
+    :hints(("Goal" :in-theory (enable upcase-char
+                                      char-fix))))
+
+  (defthm upcase-char-of-downcase-char
+    (equal (upcase-char (downcase-char x))
+           (upcase-char x))
+    :hints(("Goal" :in-theory (enable upcase-char
+                                      char-fix))))
+
   ;; Rewrite ACL2's char-downcase to downcase-char.  It seems simplest to just do
   ;; the proof by exhaustive testing.
 
@@ -388,17 +421,17 @@ characters.</p>"
 
   (defthm char-downcase-is-downcase-char
     (equal (acl2::char-downcase x)
-           (downcase-char x))))
+           (downcase-char (double-rewrite x)))))
 
 
 
 (defsection upcase-char-str
-  :parents (case-conversion)
+  :parents (cases)
   :short "Convert a character into an upper-case one-element string."
   :long "<p>@(call upcase-char-str) is logically equal to:</p>
 
 @({
- (coerce (list (upcase-char c)) 'string)
+ (implode (list (upcase-char c)))
 })
 
 <p>But we store these strings in a table so that they don't have to be
@@ -409,7 +442,7 @@ during @(see upcase-first).</p>"
     (declare (xargs :guard (and (natp n)
                                 (<= n 255))
                     :ruler-extenders :all))
-    (cons (cons n (coerce (list (upcase-char (code-char n))) 'string))
+    (cons (cons n (implode (list (upcase-char (code-char n)))))
           (if (zp n)
               nil
             (make-upcase-first-strtbl (- n 1)))))
@@ -425,7 +458,7 @@ during @(see upcase-first).</p>"
   (local (defun test (n)
            (declare (xargs :ruler-extenders :all))
            (and (equal (aref1 '*upcase-first-strtbl* *upcase-first-strtbl* n)
-                       (coerce (list (upcase-char (code-char n))) 'string))
+                       (implode (list (upcase-char (code-char n)))))
                 (if (zp n)
                     t
                   (test (- n 1))))))
@@ -437,30 +470,30 @@ during @(see upcase-first).</p>"
                          (<= n 255)
                          (test n))
                     (equal (aref1 '*upcase-first-strtbl* *upcase-first-strtbl* i)
-                           (coerce (list (upcase-char (code-char i))) 'string)))
+                           (implode (list (upcase-char (code-char i))))))
            :hints(("Goal" :induct (test n)))))
 
   (local (defthm l1
            (implies (and (natp i)
                          (<= i 255))
                     (equal (aref1 '*upcase-first-strtbl* *upcase-first-strtbl* i)
-                           (coerce (list (upcase-char (code-char i))) 'string)))
+                           (implode (list (upcase-char (code-char i))))))
            :hints(("Goal" :use ((:instance l0 (n 255)))))))
 
   (definline upcase-char-str (c)
     (declare (type character c))
-    (mbe :logic (coerce (list (upcase-char c)) 'string)
+    (mbe :logic (implode (list (upcase-char c)))
          :exec (aref1 '*upcase-first-strtbl* *upcase-first-strtbl* (char-code c)))))
 
 
 
 (defsection downcase-char-str
-  :parents (case-conversion)
+  :parents (cases)
   :short "Convert a character into a lower-case one-element string."
   :long "<p>@(call downcase-char-str) is logically equal to:</p>
 
 @({
- (coerce (list (downcase-char c)) 'string)
+ (implode (downcase-char c))
 })
 
 <p>But we store these strings in a table so that they don't have to be
@@ -471,7 +504,7 @@ during @(see downcase-first).</p>"
      (declare (xargs :guard (and (natp n)
                                  (<= n 255))
                      :ruler-extenders :all))
-    (cons (cons n (coerce (list (downcase-char (code-char n))) 'string))
+    (cons (cons n (implode (list (downcase-char (code-char n)))))
           (if (zp n)
               nil
             (make-downcase-first-strtbl (- n 1)))))
@@ -487,7 +520,7 @@ during @(see downcase-first).</p>"
   (local (defun test (n)
            (declare (xargs :ruler-extenders :all))
            (and (equal (aref1 '*downcase-first-strtbl* *downcase-first-strtbl* n)
-                       (coerce (list (downcase-char (code-char n))) 'string))
+                       (implode (list (downcase-char (code-char n)))))
                 (if (zp n)
                     t
                   (test (- n 1))))))
@@ -499,18 +532,18 @@ during @(see downcase-first).</p>"
                          (<= n 255)
                          (test n))
                     (equal (aref1 '*downcase-first-strtbl* *downcase-first-strtbl* i)
-                           (coerce (list (downcase-char (code-char i))) 'string)))
+                           (implode (list (downcase-char (code-char i))))))
            :hints(("Goal" :induct (test n)))))
 
   (local (defthm l1
            (implies (and (natp i)
                          (<= i 255))
                     (equal (aref1 '*downcase-first-strtbl* *downcase-first-strtbl* i)
-                           (coerce (list (downcase-char (code-char i))) 'string)))
+                           (implode (list (downcase-char (code-char i))))))
            :hints(("Goal" :use ((:instance l0 (n 255)))))))
 
   (definline downcase-char-str (c)
     (declare (type character c))
-    (mbe :logic (coerce (list (downcase-char c)) 'string)
+    (mbe :logic (implode (list (downcase-char c)))
          :exec (aref1 '*downcase-first-strtbl* *downcase-first-strtbl* (char-code c)))))
 
