@@ -1,5 +1,5 @@
 ; ACL2 String Library
-; Copyright (C) 2009-2010 Centaur Technology
+; Copyright (C) 2009-2013 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
@@ -19,8 +19,9 @@
 ; Original author: Jared Davis <jared@centtech.com>
 
 (in-package "STR")
+(include-book "coerce")
 (include-book "tools/bstar" :dir :system)
-(local (include-book "std/misc/explode-atom" :dir :system))
+(local (include-book "explode-atom"))
 (local (include-book "arithmetic"))
 
 (defund insert-underscores (x)
@@ -57,11 +58,15 @@
   ;;
   ;; Typical usage is (cw "foo: ~x0~%" (str::hexify foo))
   (declare (xargs :guard t))
-  (cond ((natp x)
-         (b* ((chars (explode-atom x 16)) ;; looks like BEEF...
-              (nice-chars (list* #\# #\u #\x (first chars)
-                                 (insert-underscores (nthcdr 1 chars)))))
-           (coerce nice-chars 'string)))
+  (cond ((integerp x)
+         (b* ((xsign (< x 0))
+              (xabs (abs x))
+              (chars (explode-atom xabs 16)) ;; looks like BEEF...
+              (nice-chars (list* #\# #\u #\x 
+                                 (append (and xsign '(#\-))
+                                         (cons (first chars)
+                                               (insert-underscores (nthcdr 1 chars)))))))
+           (implode nice-chars)))
         ((symbolp x)
          (symbol-name x))
         ((stringp x)
