@@ -1,5 +1,5 @@
 ; ACL2 String Library
-; Copyright (C) 2009-2010 Centaur Technology
+; Copyright (C) 2009-2013 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
@@ -21,7 +21,7 @@
 (in-package "STR")
 (include-book "iless")
 (include-book "defsort/defsort" :dir :system)
-;(include-book "std/lists/list-defunslist-fix" :dir :system)
+(local (include-book "std/typed-lists/string-listp" :dir :system))
 
 (ACL2::defsort
  :comparablep stringp
@@ -33,11 +33,6 @@
          (string-listp (list-fix x)))
   :hints(("Goal" :in-theory (enable istr-list-p))))
 
-(defthm string-listp-of-list-fix
-  ;; BOZO misplaced
-  (implies (string-listp x)
-           (string-listp (list-fix x))))
-
 (defthm string-listp-of-istr-sort
   (implies (force (string-listp x))
            (string-listp (istr-sort x)))
