@@ -1,5 +1,5 @@
 ; ACL2 String Library
-; Copyright (C) 2009-2010 Centaur Technology
+; Copyright (C) 2009-2013 Centaur Technology
 ;
 ; Contact:
 ;   Centaur Technology Formal Verification Group
@@ -74,7 +74,7 @@ details.</p>
 <h3>Copyright Information</h3>
 
 <p>ACL2 String Library<br/>
-Copyright (C) 2009-2011
+Copyright (C) 2009-2013
 <a href=\"http://www.centtech.com\">Centaur Technology</a>.</p>
 
 <p>Contact:</p>
@@ -103,11 +103,44 @@ Street, Suite 500, Boston, MA 02110-1335, USA.</p>")
 
 (defsection equivalences
   :parents (str)
-  :short "Basic equivalence relations.")
+  :short "Basic equivalence relations."
+
+  :long "<p>The string library provides the various @(see acl2::equivalence)
+relations about characters, character lists, and strings.  We end up with the
+following @(see acl2::refinement) hierarchy:</p>
+
+@({
+                      equal
+          ______________|________________
+         |              |                |
+       chareqv         list-equiv       streqv
+         |              |                |
+       ichareqv        charlisteqv      istreqv
+                        |
+                       icharlisteqv
+})")
 
 (defsection concatenation
   :parents (str)
-  :short "Functions for concatenating strings and character lists.")
+  :short "Functions for joining strings together.
+
+<p><b><color rgb='#ff0000'>Efficiency Warning</color></b>.  Concatenating
+strings in ACL2 is fundamentally slow.  Why?  In Common Lisp, strings are just
+arrays of characters, and there is not any mechanism for efficiently splicing
+together arrays.  Any kind of string concatenation, then, minimally requires
+creating a completely new array and copying all of the input characters into
+it.  This makes it especially slow to repeatedly use, e.g., @(see cat) to build
+up a string.</p>
+
+<p>To build strings more efficiently, a good general strategy is to build up a
+reverse-order character list, and then convert it into a string at the end.
+See for instance the functions @(see revappend-chars) and @(see
+rchars-to-string), which make this rather easy to do.</p>")
+
+(defsection coercion
+  :parents (str)
+  :short "Functions for converting between strings, symbols, character lists,
+and so on.")
 
 (defsection ordering
   :parents (str)
@@ -123,10 +156,10 @@ Street, Suite 500, Boston, MA 02110-1335, USA.</p>")
   :long "<p>See also @(see ordering) for some functions that can sort strings
 in alphanumeric ways.</p>")
 
-(defsection case-conversion
+(defsection cases
   :parents (str)
-  :short "Functions for recognizing upper- and lower-case characters,
-converting between cases, etc.")
+  :short "Functions for recognizing and translating between upper- and
+lower-case.")
 
 (defsection symbols
   :parents (str)
@@ -135,3 +168,18 @@ converting between cases, etc.")
 (defsection substitution
   :parents (str)
   :short "Functions for doing string replacement.")
+
+(defsection lines
+  :parents (str)
+  :short "Functions for operating on the lines of a string."
+
+  :long "<p>Note that these functions generally work with Unix-style newline
+characters, i.e., @('\\n') instead of something like @('\\r\\n').  Depending on
+your application, this may or may not be appropriate.</p>
+
+<p>One option for treating a string as lines is to just use, e.g., @(see
+strtok) to literally split it into a list of lines.  The functions here are
+generally meant to be more efficient, e.g., @(see prefix-lines) can add a
+prefix to every line without constructing an temporary string list or doing any
+intermediate string concatenation.</p>")
+
