@@ -28,10 +28,10 @@ or @(see set-raw-mode)."
 
   :long "<p>Sometimes you want to include raw Lisp code in an ACL2 book to
 achieve better performance or do fancy things like connect to external
-programs.  With @(see trust-tags), you can do this.  Unfortunately, the
-built-in mechanisms (@(see progn!) and @(see set-raw-mode)) have some
-portability problems related to compilation, file paths, read tables, non-ACL2
-objects, and so on; see below for some examples.</p>
+programs.  With <see topic='@(url defttag)'>trust tags</see>, you can do this.
+Unfortunately, the built-in mechanisms (@(see progn!) and @(see set-raw-mode))
+have some portability problems related to compilation, file paths, read tables,
+non-ACL2 objects, and so on; see below for some examples.</p>
 
 <h3>Using Include-Raw</h3>
 
@@ -195,7 +195,12 @@ differences between ACL2's reader and what raw Lisp code is expecting.</p>")
                (compiled-fname (compile-file-pathname fname)))
           (cond
            ((not (member :cltl2 *features*)) ; #-cltl2
-            (load-compiled compiled-fname))
+            (cond ((probe-file compiled-fname)
+                   (load-compiled compiled-fname))
+                  (t (format t "Compiled file ~a does not exist; loading uncompiled ~a~%"
+                             (namestring compiled-fname)
+                             fname)
+                     (raw-load-uncompiled name error-on-fail on-fail state))))
            (t ; #+cltl2
             (handler-case
              (load-compiled compiled-fname)
